=== modified file 'drizzled/Makefile.am'
--- drizzled/Makefile.am	2008-08-13 22:03:16 +0000
+++ drizzled/Makefile.am	2008-08-15 21:14:46 +0000
@@ -137,6 +137,8 @@
 		 connect.h \
 		 sql_table.h \
 		 show.h \
+		 tmp_table.h \
+		 sj_tmp_table.h \
 		 drizzled_error_messages.h \
 		 common_includes.h 
 
@@ -219,6 +221,8 @@
 		   my_decimal.cc\
 		   sql_plugin.cc \
 		   sql_binlog.cc \
+		   tmp_table.cc \
+		   sj_tmp_table.cc \
 		   sql_builtin.cc 
 
 libhandler_la_SOURCES = handler.cc

=== modified file 'drizzled/handler.cc'
--- drizzled/handler.cc	2008-08-12 21:13:01 +0000
+++ drizzled/handler.cc	2008-08-15 21:14:46 +0000
@@ -3099,7 +3099,7 @@
     goto err;
 
   if (update_create_info)
-    update_create_info_from_table(create_info, &table);
+    table.updateCreateInfo(create_info);
 
   name= check_lowercase_names(table.file, share.path.str, name_buff);
 
@@ -3168,7 +3168,7 @@
     return(3);
   }
 
-  update_create_info_from_table(&create_info, &table);
+  table.updateCreateInfo(&create_info);
   create_info.table_options|= HA_OPTION_CREATE_FROM_ENGINE;
 
   check_lowercase_names(table.file, path, path);

=== modified file 'drizzled/item_subselect.cc'
--- drizzled/item_subselect.cc	2008-08-06 18:46:02 +0000
+++ drizzled/item_subselect.cc	2008-08-15 21:14:46 +0000
@@ -2960,7 +2960,7 @@
       tmp_table->s->uniques ||
       tmp_table->key_info->key_length >= tmp_table->file->max_key_length() ||
       tmp_table->key_info->key_parts > tmp_table->file->max_key_parts());
-    free_tmp_table(thd, tmp_table);
+    tmp_table->free_tmp_table(thd);
     delete result;
     result= NULL;
     return(true);
@@ -3051,7 +3051,7 @@
 {
   delete result;
   if (tab)
-    free_tmp_table(thd, tab->table);
+    tab->table->free_tmp_table(thd);
 }
 
 

=== modified file 'drizzled/item_sum.cc'
--- drizzled/item_sum.cc	2008-08-10 06:11:44 +0000
+++ drizzled/item_sum.cc	2008-08-15 21:14:46 +0000
@@ -2488,7 +2488,7 @@
     is_evaluated= false;
     if (table)
     {
-      free_tmp_table(table->in_use, table);
+      table->free_tmp_table(table->in_use);
       table= 0;
     }
     delete tmp_table_param;
@@ -3155,7 +3155,7 @@
     if (table)
     {
       THD *thd= table->in_use;
-      free_tmp_table(thd, table);
+      table->free_tmp_table(thd);
       table= 0;
       if (tree)
       {

=== modified file 'drizzled/server_includes.h'
--- drizzled/server_includes.h	2008-08-14 22:05:07 +0000
+++ drizzled/server_includes.h	2008-08-15 21:14:46 +0000
@@ -927,7 +927,6 @@
 int create_frm(THD *thd, const char *name, const char *db, const char *table,
                uint reclength, uchar *fileinfo,
 	       HA_CREATE_INFO *create_info, uint keys, KEY *key_info);
-void update_create_info_from_table(HA_CREATE_INFO *info, TABLE *form);
 int rename_file_ext(const char * from,const char * to,const char * ext);
 bool check_db_name(LEX_STRING *db);
 bool check_column_name(const char *name);

=== added file 'drizzled/sj_tmp_table.cc'
--- drizzled/sj_tmp_table.cc	1970-01-01 00:00:00 +0000
+++ drizzled/sj_tmp_table.cc	2008-08-15 21:14:46 +0000
@@ -0,0 +1,313 @@
+#include "sj_tmp_table.h"
+
+/*
+  Create a temporary table to weed out duplicate rowid combinations
+
+  SYNOPSIS
+
+    create_duplicate_weedout_tmp_table()
+      thd
+      uniq_tuple_length_arg
+      SJ_TMP_TABLE 
+
+  DESCRIPTION
+    Create a temporary table to weed out duplicate rowid combinations. The
+    table has a single column that is a concatenation of all rowids in the
+    combination. 
+
+    Depending on the needed length, there are two cases:
+
+    1. When the length of the column < max_key_length:
+
+      CREATE TABLE tmp (col VARBINARY(n) NOT NULL, UNIQUE KEY(col));
+
+    2. Otherwise (not a valid SQL syntax but internally supported):
+
+      CREATE TABLE tmp (col VARBINARY NOT NULL, UNIQUE CONSTRAINT(col));
+
+    The code in this function was produced by extraction of relevant parts
+    from create_tmp_table().
+
+  RETURN
+    created table
+    NULL on error
+*/
+
+TABLE *create_duplicate_weedout_tmp_table(THD *thd, 
+					  uint uniq_tuple_length_arg,
+					  SJ_TMP_TABLE *sjtbl)
+{
+  MEM_ROOT *mem_root_save, own_root;
+  TABLE *table;
+  TABLE_SHARE *share;
+  uint  temp_pool_slot=MY_BIT_NONE;
+  char	*tmpname,path[FN_REFLEN];
+  Field **reg_field;
+  KEY_PART_INFO *key_part_info;
+  KEY *keyinfo;
+  uchar *group_buff;
+  uchar *bitmaps;
+  uint *blob_field;
+  MI_COLUMNDEF *recinfo, *start_recinfo;
+  bool using_unique_constraint=false;
+  Field *field, *key_field;
+  uint blob_count, null_pack_length, null_count;
+  uchar *null_flags;
+  uchar *pos;
+  
+  /*
+    STEP 1: Get temporary table name
+  */
+  statistic_increment(thd->status_var.created_tmp_tables, &LOCK_status);
+  if (use_temp_pool && !(test_flags & TEST_KEEP_TMP_TABLES))
+    temp_pool_slot = bitmap_lock_set_next(&temp_pool);
+
+  if (temp_pool_slot != MY_BIT_NONE) // we got a slot
+    sprintf(path, "%s_%lx_%i", tmp_file_prefix,
+	    current_pid, temp_pool_slot);
+  else
+  {
+    /* if we run out of slots or we are not using tempool */
+    sprintf(path,"%s%lx_%lx_%x", tmp_file_prefix,current_pid,
+            thd->thread_id, thd->tmp_table++);
+  }
+  fn_format(path, path, mysql_tmpdir, "", MY_REPLACE_EXT|MY_UNPACK_FILENAME);
+
+  /* STEP 2: Figure if we'll be using a key or blob+constraint */
+  if (uniq_tuple_length_arg >= CONVERT_IF_BIGGER_TO_BLOB)
+    using_unique_constraint= true;
+
+  /* STEP 3: Allocate memory for temptable description */
+  init_sql_alloc(&own_root, TABLE_ALLOC_BLOCK_SIZE, 0);
+  if (!multi_alloc_root(&own_root,
+                        &table, sizeof(*table),
+                        &share, sizeof(*share),
+                        &reg_field, sizeof(Field*) * (1+1),
+                        &blob_field, sizeof(uint)*2,
+                        &keyinfo, sizeof(*keyinfo),
+                        &key_part_info, sizeof(*key_part_info) * 2,
+                        &start_recinfo,
+                        sizeof(*recinfo)*(1*2+4),
+                        &tmpname, (uint) strlen(path)+1,
+                        &group_buff, (!using_unique_constraint ?
+                                      uniq_tuple_length_arg : 0),
+                        &bitmaps, bitmap_buffer_size(1)*2,
+                        NullS))
+  {
+    if (temp_pool_slot != MY_BIT_NONE)
+      bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
+    return(NULL);
+  }
+  stpcpy(tmpname,path);
+  
+
+  /* STEP 4: Create TABLE description */
+  memset(table, 0, sizeof(*table));
+  memset(reg_field, 0, sizeof(Field*)*2);
+
+  table->mem_root= own_root;
+  mem_root_save= thd->mem_root;
+  thd->mem_root= &table->mem_root;
+
+  table->field=reg_field;
+  table->alias= "weedout-tmp";
+  table->reginfo.lock_type=TL_WRITE;	/* Will be updated */
+  table->db_stat=HA_OPEN_KEYFILE+HA_OPEN_RNDFILE;
+  table->map=1;
+  table->temp_pool_slot = temp_pool_slot;
+  table->copy_blobs= 1;
+  table->in_use= thd;
+  table->quick_keys.init();
+  table->covering_keys.init();
+  table->keys_in_use_for_query.init();
+
+  table->s= share;
+  init_tmp_table_share(thd, share, "", 0, tmpname, tmpname);
+  share->blob_field= blob_field;
+  share->blob_ptr_size= portable_sizeof_char_ptr;
+  share->db_low_byte_first=1;                // True for HEAP and MyISAM
+  share->table_charset= NULL;
+  share->primary_key= MAX_KEY;               // Indicate no primary key
+  share->keys_for_keyread.init();
+  share->keys_in_use.init();
+
+  blob_count= 0;
+
+  /* Create the field */
+  {
+    /*
+      For the sake of uniformity, always use Field_varstring.
+    */
+    field= new Field_varstring(uniq_tuple_length_arg, false, "rowids", share,
+                               &my_charset_bin);
+    if (!field)
+      return(0);
+    field->table= table;
+    field->key_start.init(0);
+    field->part_of_key.init(0);
+    field->part_of_sortkey.init(0);
+    field->unireg_check= Field::NONE;
+    field->flags= (NOT_NULL_FLAG | BINARY_FLAG | NO_DEFAULT_VALUE_FLAG);
+    field->reset_fields();
+    field->init(table);
+    field->orig_table= NULL;
+     
+    field->field_index= 0;
+    
+    *(reg_field++)= field;
+    *blob_field= 0;
+    *reg_field= 0;
+
+    share->fields= 1;
+    share->blob_fields= 0;
+  }
+
+  uint reclength= field->pack_length();
+  if (using_unique_constraint)
+  { 
+    share->db_plugin= ha_lock_engine(0, myisam_hton);
+    table->file= get_new_handler(share, &table->mem_root,
+                                 share->db_type());
+    assert(uniq_tuple_length_arg <= table->file->max_key_length());
+  }
+  else
+  {
+    share->db_plugin= ha_lock_engine(0, heap_hton);
+    table->file= get_new_handler(share, &table->mem_root,
+                                 share->db_type());
+  }
+  if (!table->file)
+    goto err;
+
+  null_count=1;
+  
+  null_pack_length= 1;
+  reclength += null_pack_length;
+
+  share->reclength= reclength;
+  {
+    uint alloc_length=ALIGN_SIZE(share->reclength + MI_UNIQUE_HASH_LENGTH+1);
+    share->rec_buff_length= alloc_length;
+    if (!(table->record[0]= (uchar*)
+                            alloc_root(&table->mem_root, alloc_length*3)))
+      goto err;
+    table->record[1]= table->record[0]+alloc_length;
+    share->default_values= table->record[1]+alloc_length;
+  }
+  table->setup_tmp_table_column_bitmaps(bitmaps);
+
+  recinfo= start_recinfo;
+  null_flags=(uchar*) table->record[0];
+  pos=table->record[0]+ null_pack_length;
+  if (null_pack_length)
+  {
+    memset(recinfo, 0, sizeof(*recinfo));
+    recinfo->type=FIELD_NORMAL;
+    recinfo->length=null_pack_length;
+    recinfo++;
+    memset(null_flags, 255, null_pack_length);	// Set null fields
+
+    table->null_flags= (uchar*) table->record[0];
+    share->null_fields= null_count;
+    share->null_bytes= null_pack_length;
+  }
+  null_count=1;
+
+  {
+    //Field *field= *reg_field;
+    uint length;
+    memset(recinfo, 0, sizeof(*recinfo));
+    field->move_field(pos,(uchar*) 0,0);
+
+    field->reset();
+    /*
+      Test if there is a default field value. The test for ->ptr is to skip
+      'offset' fields generated by initalize_tables
+    */
+    // Initialize the table field:
+    memset(field->ptr, 0, field->pack_length());
+
+    length=field->pack_length();
+    pos+= length;
+
+    /* Make entry for create table */
+    recinfo->length=length;
+    if (field->flags & BLOB_FLAG)
+      recinfo->type= (int) FIELD_BLOB;
+    else
+      recinfo->type=FIELD_NORMAL;
+
+    field->table_name= &table->alias;
+  }
+
+  //param->recinfo=recinfo;
+  //store_record(table,s->default_values);        // Make empty default record
+
+  if (thd->variables.tmp_table_size == ~ (uint64_t) 0)		// No limit
+    share->max_rows= ~(ha_rows) 0;
+  else
+    share->max_rows= (ha_rows) (((share->db_type() == heap_hton) ?
+                                 min(thd->variables.tmp_table_size,
+                                     thd->variables.max_heap_table_size) :
+                                 thd->variables.tmp_table_size) /
+			         share->reclength);
+  set_if_bigger(share->max_rows,1);		// For dummy start options
+
+
+  //// keyinfo= param->keyinfo;
+  if (true)
+  {
+    share->keys=1;
+    share->uniques= test(using_unique_constraint);
+    table->key_info=keyinfo;
+    keyinfo->key_part=key_part_info;
+    keyinfo->flags=HA_NOSAME;
+    keyinfo->usable_key_parts= keyinfo->key_parts= 1;
+    keyinfo->key_length=0;
+    keyinfo->rec_per_key=0;
+    keyinfo->algorithm= HA_KEY_ALG_UNDEF;
+    keyinfo->name= (char*) "weedout_key";
+    {
+      key_part_info->null_bit=0;
+      key_part_info->field=  field;
+      key_part_info->offset= field->offset(table->record[0]);
+      key_part_info->length= (uint16_t) field->key_length();
+      key_part_info->type=   (uint8_t) field->key_type();
+      key_part_info->key_type = FIELDFLAG_BINARY;
+      if (!using_unique_constraint)
+      {
+	if (!(key_field= field->new_key_field(thd->mem_root, table,
+                                              group_buff,
+                                              field->null_ptr,
+                                              field->null_bit)))
+	  goto err;
+        key_part_info->key_part_flag|= HA_END_SPACE_ARE_EQUAL; //todo need this?
+      }
+      keyinfo->key_length+=  key_part_info->length;
+    }
+  }
+
+  if (thd->is_fatal_error)				// If end of memory
+    goto err;
+  share->db_record_offset= 1;
+  if (share->db_type() == myisam_hton)
+  {
+    recinfo++;
+    if (table->create_myisam_tmp_table(keyinfo, start_recinfo, &recinfo, 0))
+      goto err;
+  }
+  sjtbl->start_recinfo= start_recinfo;
+  sjtbl->recinfo=       recinfo;
+  if (table->open_tmp_table())
+    goto err;
+
+  thd->mem_root= mem_root_save;
+  return(table);
+
+err:
+  thd->mem_root= mem_root_save;
+  table->free_tmp_table(thd);                    /* purecov: inspected */
+  if (temp_pool_slot != MY_BIT_NONE)
+    bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
+  return(NULL);				/* purecov: inspected */
+}

=== added file 'drizzled/sj_tmp_table.h'
--- drizzled/sj_tmp_table.h	1970-01-01 00:00:00 +0000
+++ drizzled/sj_tmp_table.h	2008-08-15 21:14:46 +0000
@@ -0,0 +1,52 @@
+#ifndef DRIZZLED_SJ_TMP_TABLE_H
+#define DRIZZLED_SJ_TMP_TABLE_H
+
+#include <drizzled/server_includes.h>
+#include "table.h"
+
+/*
+  Describes use of one temporary table to weed out join duplicates.
+  The temporar
+
+  Used to
+    - create a temp table
+    - when we reach the weed-out tab, walk through rowid-ed tabs and
+      and copy rowids.
+      For each table we need
+       - rowid offset
+       - null bit address.
+*/
+
+class SJ_TMP_TABLE : public Sql_alloc
+{
+public:
+  /* Array of pointers to tables that should be "used" */
+  class TAB
+  {
+  public:
+    struct st_join_table *join_tab;
+    uint rowid_offset;
+    ushort null_byte;
+    uchar null_bit;
+  };
+  TAB *tabs;
+  TAB *tabs_end;
+
+  uint null_bits;
+  uint null_bytes;
+  uint rowid_len;
+
+  TABLE *tmp_table;
+
+  MI_COLUMNDEF *start_recinfo;
+  MI_COLUMNDEF *recinfo;
+
+  /* Pointer to next table (next->start_idx > this->end_idx) */
+  SJ_TMP_TABLE *next; 
+};
+
+TABLE *create_duplicate_weedout_tmp_table(THD *thd, 
+					  uint uniq_tuple_length_arg,
+					  SJ_TMP_TABLE *sjtbl);
+
+#endif /* DRIZZLED_SJ_TMP_TABLE_H */

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2008-08-14 22:05:07 +0000
+++ drizzled/sql_base.cc	2008-08-15 21:14:46 +0000
@@ -1023,7 +1023,7 @@
     for (table= thd->derived_tables ; table ; table= next)
     {
       next= table->next;
-      free_tmp_table(thd, table);
+      table->free_tmp_table(thd);
     }
     thd->derived_tables= 0;
   }

=== modified file 'drizzled/sql_select.cc'
--- drizzled/sql_select.cc	2008-08-15 15:01:59 +0000
+++ drizzled/sql_select.cc	2008-08-15 21:14:46 +0000
@@ -25,6 +25,7 @@
 */
 #include <drizzled/server_includes.h>
 #include <drizzled/sql_select.h>
+#include "sj_tmp_table.h"
 
 #include <mysys/my_bit.h>
 #include <drizzled/drizzled_error_messages.h>
@@ -116,11 +117,6 @@
                            List<TABLE_LIST> *join_list,
 			   Item::cond_result *cond_value);
 static bool const_expression_in_where(COND *conds,Item *item, Item **comp_item);
-static bool open_tmp_table(TABLE *table);
-static bool create_myisam_tmp_table(TABLE *table, KEY *keyinfo, 
-                                    MI_COLUMNDEF *start_recinfo,
-                                    MI_COLUMNDEF **recinfo,
-				    uint64_t options);
 static int do_select(JOIN *join,List<Item> *fields,TABLE *tmp_table);
 
 static enum_nested_loop_state
@@ -847,10 +843,6 @@
 }
 
 
-TABLE *create_duplicate_weedout_tmp_table(THD *thd, uint uniq_tuple_length_arg,
-                                          SJ_TMP_TABLE *sjtbl);
-
-
 /*
   Setup the strategies to eliminate semi-join duplicates.
   
@@ -1198,7 +1190,7 @@
   {
     if (sj_tbl->tmp_table)
     {
-      free_tmp_table(join->thd, sj_tbl->tmp_table);
+      sj_tbl->tmp_table->free_tmp_table(join->thd);
     }
   }
   join->sj_tmp_tables= NULL;
@@ -2562,9 +2554,9 @@
 
   cleanup(1);
   if (exec_tmp_table1)
-    free_tmp_table(thd, exec_tmp_table1);
+    exec_tmp_table1->free_tmp_table(thd);
   if (exec_tmp_table2)
-    free_tmp_table(thd, exec_tmp_table2);
+    exec_tmp_table2->free_tmp_table(thd);
   delete select;
   delete_dynamic(&keyuse);
   return(error);
@@ -10522,1835 +10514,6 @@
   return 0;
 }
 
-/****************************************************************************
-  Create internal temporary table
-****************************************************************************/
-
-/**
-  Create field for temporary table from given field.
-
-  @param thd	       Thread handler
-  @param org_field    field from which new field will be created
-  @param name         New field name
-  @param table	       Temporary table
-  @param item	       !=NULL if item->result_field should point to new field.
-                      This is relevant for how fill_record() is going to work:
-                      If item != NULL then fill_record() will update
-                      the record in the original table.
-                      If item == NULL then fill_record() will update
-                      the temporary table
-  @param convert_blob_length   If >0 create a varstring(convert_blob_length)
-                               field instead of blob.
-
-  @retval
-    NULL		on error
-  @retval
-    new_created field
-*/
-
-Field *create_tmp_field_from_field(THD *thd, Field *org_field,
-                                   const char *name, TABLE *table,
-                                   Item_field *item, uint convert_blob_length)
-{
-  Field *new_field;
-
-  /* 
-    Make sure that the blob fits into a Field_varstring which has 
-    2-byte lenght. 
-  */
-  if (convert_blob_length && convert_blob_length <= Field_varstring::MAX_SIZE &&
-      (org_field->flags & BLOB_FLAG))
-    new_field= new Field_varstring(convert_blob_length,
-                                   org_field->maybe_null(),
-                                   org_field->field_name, table->s,
-                                   org_field->charset());
-  else
-    new_field= org_field->new_field(thd->mem_root, table,
-                                    table == org_field->table);
-  if (new_field)
-  {
-    new_field->init(table);
-    new_field->orig_table= org_field->orig_table;
-    if (item)
-      item->result_field= new_field;
-    else
-      new_field->field_name= name;
-    new_field->flags|= (org_field->flags & NO_DEFAULT_VALUE_FLAG);
-    if (org_field->maybe_null() || (item && item->maybe_null))
-      new_field->flags&= ~NOT_NULL_FLAG;	// Because of outer join
-    if (org_field->type() == DRIZZLE_TYPE_VARCHAR)
-      table->s->db_create_options|= HA_OPTION_PACK_RECORD;
-    else if (org_field->type() == DRIZZLE_TYPE_DOUBLE)
-      ((Field_double *) new_field)->not_fixed= true;
-  }
-  return new_field;
-}
-
-/**
-  Create field for temporary table using type of given item.
-
-  @param thd                   Thread handler
-  @param item                  Item to create a field for
-  @param table                 Temporary table
-  @param copy_func             If set and item is a function, store copy of
-                               item in this array
-  @param modify_item           1 if item->result_field should point to new
-                               item. This is relevent for how fill_record()
-                               is going to work:
-                               If modify_item is 1 then fill_record() will
-                               update the record in the original table.
-                               If modify_item is 0 then fill_record() will
-                               update the temporary table
-  @param convert_blob_length   If >0 create a varstring(convert_blob_length)
-                               field instead of blob.
-
-  @retval
-    0  on error
-  @retval
-    new_created field
-*/
-
-static Field *create_tmp_field_from_item(THD *thd __attribute__((unused)),
-                                         Item *item, TABLE *table,
-                                         Item ***copy_func, bool modify_item,
-                                         uint convert_blob_length)
-{
-  bool maybe_null= item->maybe_null;
-  Field *new_field;
-
-  switch (item->result_type()) {
-  case REAL_RESULT:
-    new_field= new Field_double(item->max_length, maybe_null,
-                                item->name, item->decimals, true);
-    break;
-  case INT_RESULT:
-    /* 
-      Select an integer type with the minimal fit precision.
-      MY_INT32_NUM_DECIMAL_DIGITS is sign inclusive, don't consider the sign.
-      Values with MY_INT32_NUM_DECIMAL_DIGITS digits may or may not fit into 
-      Field_long : make them Field_int64_t.  
-    */
-    if (item->max_length >= (MY_INT32_NUM_DECIMAL_DIGITS - 1))
-      new_field=new Field_int64_t(item->max_length, maybe_null,
-                                   item->name, item->unsigned_flag);
-    else
-      new_field=new Field_long(item->max_length, maybe_null,
-                               item->name, item->unsigned_flag);
-    break;
-  case STRING_RESULT:
-    assert(item->collation.collation);
-  
-    enum enum_field_types type;
-    /*
-      DATE/TIME fields have STRING_RESULT result type. 
-      To preserve type they needed to be handled separately.
-    */
-    if ((type= item->field_type()) == DRIZZLE_TYPE_DATETIME ||
-        type == DRIZZLE_TYPE_TIME || type == DRIZZLE_TYPE_NEWDATE ||
-        type == DRIZZLE_TYPE_TIMESTAMP)
-      new_field= item->tmp_table_field_from_field_type(table, 1);
-    /* 
-      Make sure that the blob fits into a Field_varstring which has 
-      2-byte lenght. 
-    */
-    else if (item->max_length/item->collation.collation->mbmaxlen > 255 &&
-             convert_blob_length <= Field_varstring::MAX_SIZE && 
-             convert_blob_length)
-      new_field= new Field_varstring(convert_blob_length, maybe_null,
-                                     item->name, table->s,
-                                     item->collation.collation);
-    else
-      new_field= item->make_string_field(table);
-    new_field->set_derivation(item->collation.derivation);
-    break;
-  case DECIMAL_RESULT:
-  {
-    uint8_t dec= item->decimals;
-    uint8_t intg= ((Item_decimal *) item)->decimal_precision() - dec;
-    uint32_t len= item->max_length;
-
-    /*
-      Trying to put too many digits overall in a DECIMAL(prec,dec)
-      will always throw a warning. We must limit dec to
-      DECIMAL_MAX_SCALE however to prevent an assert() later.
-    */
-
-    if (dec > 0)
-    {
-      signed int overflow;
-
-      dec= min(dec, (uint8_t)DECIMAL_MAX_SCALE);
-
-      /*
-        If the value still overflows the field with the corrected dec,
-        we'll throw out decimals rather than integers. This is still
-        bad and of course throws a truncation warning.
-        +1: for decimal point
-      */
-
-      overflow= my_decimal_precision_to_length(intg + dec, dec,
-                                               item->unsigned_flag) - len;
-
-      if (overflow > 0)
-        dec= max(0, dec - overflow);            // too long, discard fract
-      else
-        len -= item->decimals - dec;            // corrected value fits
-    }
-
-    new_field= new Field_new_decimal(len, maybe_null, item->name,
-                                     dec, item->unsigned_flag);
-    break;
-  }
-  case ROW_RESULT:
-  default:
-    // This case should never be choosen
-    assert(0);
-    new_field= 0;
-    break;
-  }
-  if (new_field)
-    new_field->init(table);
-    
-  if (copy_func && item->is_result_field())
-    *((*copy_func)++) = item;			// Save for copy_funcs
-  if (modify_item)
-    item->set_result_field(new_field);
-  if (item->type() == Item::NULL_ITEM)
-    new_field->is_created_from_null_item= true;
-  return new_field;
-}
-
-
-/**
-  Create field for information schema table.
-
-  @param thd		Thread handler
-  @param table		Temporary table
-  @param item		Item to create a field for
-
-  @retval
-    0			on error
-  @retval
-    new_created field
-*/
-
-Field *create_tmp_field_for_schema(THD *thd __attribute__((unused)),
-                                   Item *item, TABLE *table)
-{
-  if (item->field_type() == DRIZZLE_TYPE_VARCHAR)
-  {
-    Field *field;
-    if (item->max_length > MAX_FIELD_VARCHARLENGTH)
-      field= new Field_blob(item->max_length, item->maybe_null,
-                            item->name, item->collation.collation);
-    else
-      field= new Field_varstring(item->max_length, item->maybe_null,
-                                 item->name,
-                                 table->s, item->collation.collation);
-    if (field)
-      field->init(table);
-    return field;
-  }
-  return item->tmp_table_field_from_field_type(table, 0);
-}
-
-
-/**
-  Create field for temporary table.
-
-  @param thd		Thread handler
-  @param table		Temporary table
-  @param item		Item to create a field for
-  @param type		Type of item (normally item->type)
-  @param copy_func	If set and item is a function, store copy of item
-                       in this array
-  @param from_field    if field will be created using other field as example,
-                       pointer example field will be written here
-  @param default_field	If field has a default value field, store it here
-  @param group		1 if we are going to do a relative group by on result
-  @param modify_item	1 if item->result_field should point to new item.
-                       This is relevent for how fill_record() is going to
-                       work:
-                       If modify_item is 1 then fill_record() will update
-                       the record in the original table.
-                       If modify_item is 0 then fill_record() will update
-                       the temporary table
-  @param convert_blob_length If >0 create a varstring(convert_blob_length)
-                             field instead of blob.
-
-  @retval
-    0			on error
-  @retval
-    new_created field
-*/
-
-Field *create_tmp_field(THD *thd, TABLE *table,Item *item, Item::Type type,
-                        Item ***copy_func, Field **from_field,
-                        Field **default_field,
-                        bool group, bool modify_item,
-                        bool table_cant_handle_bit_fields __attribute__((unused)),
-                        bool make_copy_field,
-                        uint convert_blob_length)
-{
-  Field *result;
-  Item::Type orig_type= type;
-  Item *orig_item= 0;
-
-  if (type != Item::FIELD_ITEM &&
-      item->real_item()->type() == Item::FIELD_ITEM)
-  {
-    orig_item= item;
-    item= item->real_item();
-    type= Item::FIELD_ITEM;
-  }
-
-  switch (type) {
-  case Item::SUM_FUNC_ITEM:
-  {
-    Item_sum *item_sum=(Item_sum*) item;
-    result= item_sum->create_tmp_field(group, table, convert_blob_length);
-    if (!result)
-      my_error(ER_OUT_OF_RESOURCES, MYF(ME_FATALERROR));
-    return result;
-  }
-  case Item::FIELD_ITEM:
-  case Item::DEFAULT_VALUE_ITEM:
-  {
-    Item_field *field= (Item_field*) item;
-    bool orig_modify= modify_item;
-    if (orig_type == Item::REF_ITEM)
-      modify_item= 0;
-    /*
-      If item have to be able to store NULLs but underlaid field can't do it,
-      create_tmp_field_from_field() can't be used for tmp field creation.
-    */
-    if (field->maybe_null && !field->field->maybe_null())
-    {
-      result= create_tmp_field_from_item(thd, item, table, NULL,
-                                         modify_item, convert_blob_length);
-      *from_field= field->field;
-      if (result && modify_item)
-        field->result_field= result;
-    } 
-    else
-      result= create_tmp_field_from_field(thd, (*from_field= field->field),
-                                          orig_item ? orig_item->name :
-                                          item->name,
-                                          table,
-                                          modify_item ? field :
-                                          NULL,
-                                          convert_blob_length);
-    if (orig_type == Item::REF_ITEM && orig_modify)
-      ((Item_ref*)orig_item)->set_result_field(result);
-    if (field->field->eq_def(result))
-      *default_field= field->field;
-    return result;
-  }
-  /* Fall through */
-  case Item::FUNC_ITEM:
-    /* Fall through */
-  case Item::COND_ITEM:
-  case Item::FIELD_AVG_ITEM:
-  case Item::FIELD_STD_ITEM:
-  case Item::SUBSELECT_ITEM:
-    /* The following can only happen with 'CREATE TABLE ... SELECT' */
-  case Item::PROC_ITEM:
-  case Item::INT_ITEM:
-  case Item::REAL_ITEM:
-  case Item::DECIMAL_ITEM:
-  case Item::STRING_ITEM:
-  case Item::REF_ITEM:
-  case Item::NULL_ITEM:
-  case Item::VARBIN_ITEM:
-    if (make_copy_field)
-    {
-      assert(((Item_result_field*)item)->result_field);
-      *from_field= ((Item_result_field*)item)->result_field;
-    }
-    return create_tmp_field_from_item(thd, item, table,
-                                      (make_copy_field ? 0 : copy_func),
-                                       modify_item, convert_blob_length);
-  case Item::TYPE_HOLDER:  
-    result= ((Item_type_holder *)item)->make_field_by_type(table);
-    result->set_derivation(item->collation.derivation);
-    return result;
-  default:					// Dosen't have to be stored
-    return 0;
-  }
-}
-
-/*
-  Set up column usage bitmaps for a temporary table
-
-  IMPLEMENTATION
-    For temporary tables, we need one bitmap with all columns set and
-    a tmp_set bitmap to be used by things like filesort.
-*/
-
-void setup_tmp_table_column_bitmaps(TABLE *table, uchar *bitmaps)
-{
-  uint field_count= table->s->fields;
-  bitmap_init(&table->def_read_set, (my_bitmap_map*) bitmaps, field_count,
-              false);
-  bitmap_init(&table->tmp_set,
-              (my_bitmap_map*) (bitmaps+ bitmap_buffer_size(field_count)),
-              field_count, false);
-  /* write_set and all_set are copies of read_set */
-  table->def_write_set= table->def_read_set;
-  table->s->all_set= table->def_read_set;
-  bitmap_set_all(&table->s->all_set);
-  table->default_column_bitmaps();
-}
-
-
-/**
-  Create a temp table according to a field list.
-
-  Given field pointers are changed to point at tmp_table for
-  send_fields. The table object is self contained: it's
-  allocated in its own memory root, as well as Field objects
-  created for table columns.
-  This function will replace Item_sum items in 'fields' list with
-  corresponding Item_field items, pointing at the fields in the
-  temporary table, unless this was prohibited by true
-  value of argument save_sum_fields. The Item_field objects
-  are created in THD memory root.
-
-  @param thd                  thread handle
-  @param param                a description used as input to create the table
-  @param fields               list of items that will be used to define
-                              column types of the table (also see NOTES)
-  @param group                TODO document
-  @param distinct             should table rows be distinct
-  @param save_sum_fields      see NOTES
-  @param select_options
-  @param rows_limit
-  @param table_alias          possible name of the temporary table that can
-                              be used for name resolving; can be "".
-*/
-
-#define STRING_TOTAL_LENGTH_TO_PACK_ROWS 128
-#define AVG_STRING_LENGTH_TO_PACK_ROWS   64
-#define RATIO_TO_PACK_ROWS	       2
-#define MIN_STRING_LENGTH_TO_PACK_ROWS   10
-
-TABLE *
-create_tmp_table(THD *thd,TMP_TABLE_PARAM *param,List<Item> &fields,
-		 ORDER *group, bool distinct, bool save_sum_fields,
-		 uint64_t select_options, ha_rows rows_limit,
-		 char *table_alias)
-{
-  MEM_ROOT *mem_root_save, own_root;
-  TABLE *table;
-  TABLE_SHARE *share;
-  uint	i,field_count,null_count,null_pack_length;
-  uint  copy_func_count= param->func_count;
-  uint  hidden_null_count, hidden_null_pack_length, hidden_field_count;
-  uint  blob_count,group_null_items, string_count;
-  uint  temp_pool_slot=MY_BIT_NONE;
-  uint fieldnr= 0;
-  ulong reclength, string_total_length;
-  bool  using_unique_constraint= 0;
-  bool  use_packed_rows= 0;
-  bool  not_all_columns= !(select_options & TMP_TABLE_ALL_COLUMNS);
-  char  *tmpname,path[FN_REFLEN];
-  uchar	*pos, *group_buff, *bitmaps;
-  uchar *null_flags;
-  Field **reg_field, **from_field, **default_field;
-  uint *blob_field;
-  Copy_field *copy=0;
-  KEY *keyinfo;
-  KEY_PART_INFO *key_part_info;
-  Item **copy_func;
-  MI_COLUMNDEF *recinfo;
-  uint total_uneven_bit_length= 0;
-  bool force_copy_fields= param->force_copy_fields;
-
-  status_var_increment(thd->status_var.created_tmp_tables);
-
-  if (use_temp_pool && !(test_flags & TEST_KEEP_TMP_TABLES))
-    temp_pool_slot = bitmap_lock_set_next(&temp_pool);
-
-  if (temp_pool_slot != MY_BIT_NONE) // we got a slot
-    sprintf(path, "%s_%lx_%i", tmp_file_prefix,
-            current_pid, temp_pool_slot);
-  else
-  {
-    /* if we run out of slots or we are not using tempool */
-    sprintf(path,"%s%lx_%lx_%x", tmp_file_prefix,current_pid,
-            thd->thread_id, thd->tmp_table++);
-  }
-
-  /*
-    No need to change table name to lower case as we are only creating
-    MyISAM or HEAP tables here
-  */
-  fn_format(path, path, mysql_tmpdir, "", MY_REPLACE_EXT|MY_UNPACK_FILENAME);
-
-
-  if (group)
-  {
-    if (!param->quick_group)
-      group=0;					// Can't use group key
-    else for (ORDER *tmp=group ; tmp ; tmp=tmp->next)
-    {
-      /*
-        marker == 4 means two things:
-        - store NULLs in the key, and
-        - convert BIT fields to 64-bit long, needed because MEMORY tables
-          can't index BIT fields.
-      */
-      (*tmp->item)->marker= 4;
-      if ((*tmp->item)->max_length >= CONVERT_IF_BIGGER_TO_BLOB)
-	using_unique_constraint=1;
-    }
-    if (param->group_length >= MAX_BLOB_WIDTH)
-      using_unique_constraint=1;
-    if (group)
-      distinct=0;				// Can't use distinct
-  }
-
-  field_count=param->field_count+param->func_count+param->sum_func_count;
-  hidden_field_count=param->hidden_field_count;
-
-  /*
-    When loose index scan is employed as access method, it already
-    computes all groups and the result of all aggregate functions. We
-    make space for the items of the aggregate function in the list of
-    functions TMP_TABLE_PARAM::items_to_copy, so that the values of
-    these items are stored in the temporary table.
-  */
-  if (param->precomputed_group_by)
-    copy_func_count+= param->sum_func_count;
-  
-  init_sql_alloc(&own_root, TABLE_ALLOC_BLOCK_SIZE, 0);
-
-  if (!multi_alloc_root(&own_root,
-                        &table, sizeof(*table),
-                        &share, sizeof(*share),
-                        &reg_field, sizeof(Field*) * (field_count+1),
-                        &default_field, sizeof(Field*) * (field_count),
-                        &blob_field, sizeof(uint)*(field_count+1),
-                        &from_field, sizeof(Field*)*field_count,
-                        &copy_func, sizeof(*copy_func)*(copy_func_count+1),
-                        &param->keyinfo, sizeof(*param->keyinfo),
-                        &key_part_info,
-                        sizeof(*key_part_info)*(param->group_parts+1),
-                        &param->start_recinfo,
-                        sizeof(*param->recinfo)*(field_count*2+4),
-                        &tmpname, (uint) strlen(path)+1,
-                        &group_buff, (group && ! using_unique_constraint ?
-                                      param->group_length : 0),
-                        &bitmaps, bitmap_buffer_size(field_count)*2,
-                        NullS))
-  {
-    if (temp_pool_slot != MY_BIT_NONE)
-      bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
-    return(NULL);				/* purecov: inspected */
-  }
-  /* Copy_field belongs to TMP_TABLE_PARAM, allocate it in THD mem_root */
-  if (!(param->copy_field= copy= new (thd->mem_root) Copy_field[field_count]))
-  {
-    if (temp_pool_slot != MY_BIT_NONE)
-      bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
-    free_root(&own_root, MYF(0));               /* purecov: inspected */
-    return(NULL);				/* purecov: inspected */
-  }
-  param->items_to_copy= copy_func;
-  stpcpy(tmpname,path);
-  /* make table according to fields */
-
-  memset(table, 0, sizeof(*table));
-  memset(reg_field, 0, sizeof(Field*)*(field_count+1));
-  memset(default_field, 0, sizeof(Field*) * (field_count));
-  memset(from_field, 0, sizeof(Field*)*field_count);
-
-  table->mem_root= own_root;
-  mem_root_save= thd->mem_root;
-  thd->mem_root= &table->mem_root;
-
-  table->field=reg_field;
-  table->alias= table_alias;
-  table->reginfo.lock_type=TL_WRITE;	/* Will be updated */
-  table->db_stat=HA_OPEN_KEYFILE+HA_OPEN_RNDFILE;
-  table->map=1;
-  table->temp_pool_slot = temp_pool_slot;
-  table->copy_blobs= 1;
-  table->in_use= thd;
-  table->quick_keys.init();
-  table->covering_keys.init();
-  table->keys_in_use_for_query.init();
-
-  table->setShare(share);
-  init_tmp_table_share(thd, share, "", 0, tmpname, tmpname);
-  share->blob_field= blob_field;
-  share->blob_ptr_size= portable_sizeof_char_ptr;
-  share->db_low_byte_first=1;                // True for HEAP and MyISAM
-  share->table_charset= param->table_charset;
-  share->primary_key= MAX_KEY;               // Indicate no primary key
-  share->keys_for_keyread.init();
-  share->keys_in_use.init();
-
-  /* Calculate which type of fields we will store in the temporary table */
-
-  reclength= string_total_length= 0;
-  blob_count= string_count= null_count= hidden_null_count= group_null_items= 0;
-  param->using_indirect_summary_function=0;
-
-  List_iterator_fast<Item> li(fields);
-  Item *item;
-  Field **tmp_from_field=from_field;
-  while ((item=li++))
-  {
-    Item::Type type=item->type();
-    if (not_all_columns)
-    {
-      if (item->with_sum_func && type != Item::SUM_FUNC_ITEM)
-      {
-        if (item->used_tables() & OUTER_REF_TABLE_BIT)
-          item->update_used_tables();
-        if (type == Item::SUBSELECT_ITEM ||
-            (item->used_tables() & ~OUTER_REF_TABLE_BIT))
-        {
-	  /*
-	    Mark that the we have ignored an item that refers to a summary
-	    function. We need to know this if someone is going to use
-	    DISTINCT on the result.
-	  */
-	  param->using_indirect_summary_function=1;
-	  continue;
-        }
-      }
-      if (item->const_item() && (int) hidden_field_count <= 0)
-        continue; // We don't have to store this
-    }
-    if (type == Item::SUM_FUNC_ITEM && !group && !save_sum_fields)
-    {						/* Can't calc group yet */
-      ((Item_sum*) item)->result_field=0;
-      for (i=0 ; i < ((Item_sum*) item)->arg_count ; i++)
-      {
-	Item **argp= ((Item_sum*) item)->args + i;
-	Item *arg= *argp;
-	if (!arg->const_item())
-	{
-	  Field *new_field=
-            create_tmp_field(thd, table, arg, arg->type(), &copy_func,
-                             tmp_from_field, &default_field[fieldnr],
-                             group != 0,not_all_columns,
-                             distinct, 0,
-                             param->convert_blob_length);
-	  if (!new_field)
-	    goto err;					// Should be OOM
-	  tmp_from_field++;
-	  reclength+=new_field->pack_length();
-	  if (new_field->flags & BLOB_FLAG)
-	  {
-	    *blob_field++= fieldnr;
-	    blob_count++;
-	  }
-	  *(reg_field++)= new_field;
-          if (new_field->real_type() == DRIZZLE_TYPE_VARCHAR)
-          {
-            string_count++;
-            string_total_length+= new_field->pack_length();
-          }
-          thd->mem_root= mem_root_save;
-          thd->change_item_tree(argp, new Item_field(new_field));
-          thd->mem_root= &table->mem_root;
-	  if (!(new_field->flags & NOT_NULL_FLAG))
-          {
-	    null_count++;
-            /*
-              new_field->maybe_null() is still false, it will be
-              changed below. But we have to setup Item_field correctly
-            */
-            (*argp)->maybe_null=1;
-          }
-          new_field->field_index= fieldnr++;
-	}
-      }
-    }
-    else
-    {
-      /*
-	The last parameter to create_tmp_field() is a bit tricky:
-
-	We need to set it to 0 in union, to get fill_record() to modify the
-	temporary table.
-	We need to set it to 1 on multi-table-update and in select to
-	write rows to the temporary table.
-	We here distinguish between UNION and multi-table-updates by the fact
-	that in the later case group is set to the row pointer.
-      */
-      Field *new_field= (param->schema_table) ?
-        create_tmp_field_for_schema(thd, item, table) :
-        create_tmp_field(thd, table, item, type, &copy_func,
-                         tmp_from_field, &default_field[fieldnr],
-                         group != 0,
-                         !force_copy_fields &&
-                           (not_all_columns || group !=0),
-                         /*
-                           If item->marker == 4 then we force create_tmp_field
-                           to create a 64-bit longs for BIT fields because HEAP
-                           tables can't index BIT fields directly. We do the same
-                           for distinct, as we want the distinct index to be
-                           usable in this case too.
-                         */
-                         item->marker == 4 || param->bit_fields_as_long,
-                         force_copy_fields,
-                         param->convert_blob_length);
-
-      if (!new_field)
-      {
-	if (thd->is_fatal_error)
-	  goto err;				// Got OOM
-	continue;				// Some kindf of const item
-      }
-      if (type == Item::SUM_FUNC_ITEM)
-	((Item_sum *) item)->result_field= new_field;
-      tmp_from_field++;
-      reclength+=new_field->pack_length();
-      if (!(new_field->flags & NOT_NULL_FLAG))
-	null_count++;
-      if (new_field->flags & BLOB_FLAG)
-      {
-        *blob_field++= fieldnr;
-	blob_count++;
-      }
-      if (item->marker == 4 && item->maybe_null)
-      {
-	group_null_items++;
-	new_field->flags|= GROUP_FLAG;
-      }
-      new_field->field_index= fieldnr++;
-      *(reg_field++)= new_field;
-    }
-    if (!--hidden_field_count)
-    {
-      /*
-        This was the last hidden field; Remember how many hidden fields could
-        have null
-      */
-      hidden_null_count=null_count;
-      /*
-	We need to update hidden_field_count as we may have stored group
-	functions with constant arguments
-      */
-      param->hidden_field_count= fieldnr;
-      null_count= 0;
-    }
-  }
-  assert(fieldnr == (uint) (reg_field - table->field));
-  assert(field_count >= (uint) (reg_field - table->field));
-  field_count= fieldnr;
-  *reg_field= 0;
-  *blob_field= 0;				// End marker
-  share->fields= field_count;
-
-  /* If result table is small; use a heap */
-  /* future: storage engine selection can be made dynamic? */
-  if (blob_count || using_unique_constraint ||
-      (select_options & (OPTION_BIG_TABLES | SELECT_SMALL_RESULT)) ==
-      OPTION_BIG_TABLES || (select_options & TMP_TABLE_FORCE_MYISAM))
-  {
-    share->db_plugin= ha_lock_engine(0, myisam_hton);
-    table->file= get_new_handler(share, &table->mem_root,
-                                 share->db_type());
-    if (group &&
-	(param->group_parts > table->file->max_key_parts() ||
-	 param->group_length > table->file->max_key_length()))
-      using_unique_constraint=1;
-  }
-  else
-  {
-    share->db_plugin= ha_lock_engine(0, heap_hton);
-    table->file= get_new_handler(share, &table->mem_root,
-                                 share->db_type());
-  }
-  if (!table->file)
-    goto err;
-
-
-  if (!using_unique_constraint)
-    reclength+= group_null_items;	// null flag is stored separately
-
-  share->blob_fields= blob_count;
-  if (blob_count == 0)
-  {
-    /* We need to ensure that first byte is not 0 for the delete link */
-    if (param->hidden_field_count)
-      hidden_null_count++;
-    else
-      null_count++;
-  }
-  hidden_null_pack_length=(hidden_null_count+7)/8;
-  null_pack_length= (hidden_null_pack_length +
-                     (null_count + total_uneven_bit_length + 7) / 8);
-  reclength+=null_pack_length;
-  if (!reclength)
-    reclength=1;				// Dummy select
-  /* Use packed rows if there is blobs or a lot of space to gain */
-  if (blob_count || ((string_total_length >= STRING_TOTAL_LENGTH_TO_PACK_ROWS) && (reclength / string_total_length <= RATIO_TO_PACK_ROWS || (string_total_length / string_count) >= AVG_STRING_LENGTH_TO_PACK_ROWS)))
-    use_packed_rows= 1;
-
-  share->reclength= reclength;
-  {
-    uint alloc_length=ALIGN_SIZE(reclength+MI_UNIQUE_HASH_LENGTH+1);
-    share->rec_buff_length= alloc_length;
-    if (!(table->record[0]= (uchar*)
-                            alloc_root(&table->mem_root, alloc_length*3)))
-      goto err;
-    table->record[1]= table->record[0]+alloc_length;
-    share->default_values= table->record[1]+alloc_length;
-  }
-  copy_func[0]=0;				// End marker
-  param->func_count= copy_func - param->items_to_copy; 
-
-  setup_tmp_table_column_bitmaps(table, bitmaps);
-
-  recinfo=param->start_recinfo;
-  null_flags=(uchar*) table->record[0];
-  pos=table->record[0]+ null_pack_length;
-  if (null_pack_length)
-  {
-    memset(recinfo, 0, sizeof(*recinfo));
-    recinfo->type=FIELD_NORMAL;
-    recinfo->length=null_pack_length;
-    recinfo++;
-    memset(null_flags, 255, null_pack_length);	// Set null fields
-
-    table->null_flags= (uchar*) table->record[0];
-    share->null_fields= null_count+ hidden_null_count;
-    share->null_bytes= null_pack_length;
-  }
-  null_count= (blob_count == 0) ? 1 : 0;
-  hidden_field_count=param->hidden_field_count;
-  for (i=0,reg_field=table->field; i < field_count; i++,reg_field++,recinfo++)
-  {
-    Field *field= *reg_field;
-    uint length;
-    memset(recinfo, 0, sizeof(*recinfo));
-
-    if (!(field->flags & NOT_NULL_FLAG))
-    {
-      if (field->flags & GROUP_FLAG && !using_unique_constraint)
-      {
-	/*
-	  We have to reserve one byte here for NULL bits,
-	  as this is updated by 'end_update()'
-	*/
-	*pos++=0;				// Null is stored here
-	recinfo->length=1;
-	recinfo->type=FIELD_NORMAL;
-	recinfo++;
-	memset(recinfo, 0, sizeof(*recinfo));
-      }
-      else
-      {
-	recinfo->null_bit= 1 << (null_count & 7);
-	recinfo->null_pos= null_count/8;
-      }
-      field->move_field(pos,null_flags+null_count/8,
-			1 << (null_count & 7));
-      null_count++;
-    }
-    else
-      field->move_field(pos,(uchar*) 0,0);
-    field->reset();
-
-    /*
-      Test if there is a default field value. The test for ->ptr is to skip
-      'offset' fields generated by initalize_tables
-    */
-    if (default_field[i] && default_field[i]->ptr)
-    {
-      /* 
-         default_field[i] is set only in the cases  when 'field' can
-         inherit the default value that is defined for the field referred
-         by the Item_field object from which 'field' has been created.
-      */
-      my_ptrdiff_t diff;
-      Field *orig_field= default_field[i];
-      /* Get the value from default_values */
-      diff= (my_ptrdiff_t) (orig_field->table->s->default_values-
-                            orig_field->table->record[0]);
-      orig_field->move_field_offset(diff);      // Points now at default_values
-      if (orig_field->is_real_null())
-        field->set_null();
-      else
-      {
-        field->set_notnull();
-        memcpy(field->ptr, orig_field->ptr, field->pack_length());
-      }
-      orig_field->move_field_offset(-diff);     // Back to record[0]
-    } 
-
-    if (from_field[i])
-    {						/* Not a table Item */
-      copy->set(field,from_field[i],save_sum_fields);
-      copy++;
-    }
-    length=field->pack_length();
-    pos+= length;
-
-    /* Make entry for create table */
-    recinfo->length=length;
-    if (field->flags & BLOB_FLAG)
-      recinfo->type= (int) FIELD_BLOB;
-    else
-      recinfo->type=FIELD_NORMAL;
-    if (!--hidden_field_count)
-      null_count=(null_count+7) & ~7;		// move to next byte
-
-    // fix table name in field entry
-    field->table_name= &table->alias;
-  }
-
-  param->copy_field_end=copy;
-  param->recinfo=recinfo;
-  store_record(table,s->default_values);        // Make empty default record
-
-  if (thd->variables.tmp_table_size == ~ (uint64_t) 0)		// No limit
-    share->max_rows= ~(ha_rows) 0;
-  else
-    share->max_rows= (ha_rows) (((share->db_type() == heap_hton) ?
-                                 min(thd->variables.tmp_table_size,
-                                     thd->variables.max_heap_table_size) :
-                                 thd->variables.tmp_table_size) /
-			         share->reclength);
-  set_if_bigger(share->max_rows,1);		// For dummy start options
-  /*
-    Push the LIMIT clause to the temporary table creation, so that we
-    materialize only up to 'rows_limit' records instead of all result records.
-  */
-  set_if_smaller(share->max_rows, rows_limit);
-  param->end_write_records= rows_limit;
-
-  keyinfo= param->keyinfo;
-
-  if (group)
-  {
-    table->group=group;				/* Table is grouped by key */
-    param->group_buff=group_buff;
-    share->keys=1;
-    share->uniques= test(using_unique_constraint);
-    table->key_info=keyinfo;
-    keyinfo->key_part=key_part_info;
-    keyinfo->flags=HA_NOSAME;
-    keyinfo->usable_key_parts=keyinfo->key_parts= param->group_parts;
-    keyinfo->key_length=0;
-    keyinfo->rec_per_key=0;
-    keyinfo->algorithm= HA_KEY_ALG_UNDEF;
-    keyinfo->name= (char*) "group_key";
-    ORDER *cur_group= group;
-    for (; cur_group ; cur_group= cur_group->next, key_part_info++)
-    {
-      Field *field=(*cur_group->item)->get_tmp_table_field();
-      bool maybe_null=(*cur_group->item)->maybe_null;
-      key_part_info->null_bit=0;
-      key_part_info->field=  field;
-      key_part_info->offset= field->offset(table->record[0]);
-      key_part_info->length= (uint16_t) field->key_length();
-      key_part_info->type=   (uint8_t) field->key_type();
-      key_part_info->key_type =
-	((ha_base_keytype) key_part_info->type == HA_KEYTYPE_TEXT ||
-	 (ha_base_keytype) key_part_info->type == HA_KEYTYPE_VARTEXT1 ||
-	 (ha_base_keytype) key_part_info->type == HA_KEYTYPE_VARTEXT2) ?
-	0 : FIELDFLAG_BINARY;
-      if (!using_unique_constraint)
-      {
-	cur_group->buff=(char*) group_buff;
-	if (!(cur_group->field= field->new_key_field(thd->mem_root,table,
-                                                     group_buff +
-                                                     test(maybe_null),
-                                                     field->null_ptr,
-                                                     field->null_bit)))
-	  goto err; /* purecov: inspected */
-	if (maybe_null)
-	{
-	  /*
-	    To be able to group on NULL, we reserved place in group_buff
-	    for the NULL flag just before the column. (see above).
-	    The field data is after this flag.
-	    The NULL flag is updated in 'end_update()' and 'end_write()'
-	  */
-	  keyinfo->flags|= HA_NULL_ARE_EQUAL;	// def. that NULL == NULL
-	  key_part_info->null_bit=field->null_bit;
-	  key_part_info->null_offset= (uint) (field->null_ptr -
-					      (uchar*) table->record[0]);
-          cur_group->buff++;                        // Pointer to field data
-	  group_buff++;                         // Skipp null flag
-	}
-        /* In GROUP BY 'a' and 'a ' are equal for VARCHAR fields */
-        key_part_info->key_part_flag|= HA_END_SPACE_ARE_EQUAL;
-	group_buff+= cur_group->field->pack_length();
-      }
-      keyinfo->key_length+=  key_part_info->length;
-    }
-  }
-
-  if (distinct && field_count != param->hidden_field_count)
-  {
-    /*
-      Create an unique key or an unique constraint over all columns
-      that should be in the result.  In the temporary table, there are
-      'param->hidden_field_count' extra columns, whose null bits are stored
-      in the first 'hidden_null_pack_length' bytes of the row.
-    */
-    if (blob_count)
-    {
-      /*
-        Special mode for index creation in MyISAM used to support unique
-        indexes on blobs with arbitrary length. Such indexes cannot be
-        used for lookups.
-      */
-      share->uniques= 1;
-    }
-    null_pack_length-=hidden_null_pack_length;
-    keyinfo->key_parts= ((field_count-param->hidden_field_count)+
-			 (share->uniques ? test(null_pack_length) : 0));
-    table->distinct= 1;
-    share->keys= 1;
-    if (!(key_part_info= (KEY_PART_INFO*)
-          alloc_root(&table->mem_root,
-                     keyinfo->key_parts * sizeof(KEY_PART_INFO))))
-      goto err;
-    memset(key_part_info, 0, keyinfo->key_parts * sizeof(KEY_PART_INFO));
-    table->key_info=keyinfo;
-    keyinfo->key_part=key_part_info;
-    keyinfo->flags=HA_NOSAME | HA_NULL_ARE_EQUAL;
-    keyinfo->key_length=(uint16_t) reclength;
-    keyinfo->name= (char*) "distinct_key";
-    keyinfo->algorithm= HA_KEY_ALG_UNDEF;
-    keyinfo->rec_per_key=0;
-
-    /*
-      Create an extra field to hold NULL bits so that unique indexes on
-      blobs can distinguish NULL from 0. This extra field is not needed
-      when we do not use UNIQUE indexes for blobs.
-    */
-    if (null_pack_length && share->uniques)
-    {
-      key_part_info->null_bit=0;
-      key_part_info->offset=hidden_null_pack_length;
-      key_part_info->length=null_pack_length;
-      key_part_info->field= new Field_varstring(table->record[0],
-                                                (uint32_t) key_part_info->length,
-                                                0,
-                                                (uchar*) 0,
-                                                (uint) 0,
-                                                Field::NONE,
-                                                NullS, 
-                                                table->s,
-                                                &my_charset_bin);
-      if (!key_part_info->field)
-        goto err;
-      key_part_info->field->init(table);
-      key_part_info->key_type=FIELDFLAG_BINARY;
-      key_part_info->type=    HA_KEYTYPE_BINARY;
-      key_part_info++;
-    }
-    /* Create a distinct key over the columns we are going to return */
-    for (i=param->hidden_field_count, reg_field=table->field + i ;
-	 i < field_count;
-	 i++, reg_field++, key_part_info++)
-    {
-      key_part_info->null_bit=0;
-      key_part_info->field=    *reg_field;
-      key_part_info->offset=   (*reg_field)->offset(table->record[0]);
-      key_part_info->length=   (uint16_t) (*reg_field)->pack_length();
-      /* TODO:
-        The below method of computing the key format length of the
-        key part is a copy/paste from opt_range.cc, and table.cc.
-        This should be factored out, e.g. as a method of Field.
-        In addition it is not clear if any of the Field::*_length
-        methods is supposed to compute the same length. If so, it
-        might be reused.
-      */
-      key_part_info->store_length= key_part_info->length;
-
-      if ((*reg_field)->real_maybe_null())
-        key_part_info->store_length+= HA_KEY_NULL_LENGTH;
-      if ((*reg_field)->type() == DRIZZLE_TYPE_BLOB || 
-          (*reg_field)->real_type() == DRIZZLE_TYPE_VARCHAR)
-        key_part_info->store_length+= HA_KEY_BLOB_LENGTH;
-
-      key_part_info->type=     (uint8_t) (*reg_field)->key_type();
-      key_part_info->key_type =
-	((ha_base_keytype) key_part_info->type == HA_KEYTYPE_TEXT ||
-	 (ha_base_keytype) key_part_info->type == HA_KEYTYPE_VARTEXT1 ||
-	 (ha_base_keytype) key_part_info->type == HA_KEYTYPE_VARTEXT2) ?
-	0 : FIELDFLAG_BINARY;
-    }
-  }
-
-  if (thd->is_fatal_error)				// If end of memory
-    goto err;					 /* purecov: inspected */
-  share->db_record_offset= 1;
-  if (share->db_type() == myisam_hton)
-  {
-    if (create_myisam_tmp_table(table, param->keyinfo, param->start_recinfo,
-                                &param->recinfo, select_options))
-      goto err;
-  }
-  if (open_tmp_table(table))
-    goto err;
-
-  thd->mem_root= mem_root_save;
-
-  return(table);
-
-err:
-  thd->mem_root= mem_root_save;
-  free_tmp_table(thd,table);                    /* purecov: inspected */
-  if (temp_pool_slot != MY_BIT_NONE)
-    bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
-  return(NULL);				/* purecov: inspected */
-}
-
-
-
-
-/*
-  Create a temporary table to weed out duplicate rowid combinations
-
-  SYNOPSIS
-
-    create_duplicate_weedout_tmp_table()
-      thd
-      uniq_tuple_length_arg
-      SJ_TMP_TABLE 
-
-  DESCRIPTION
-    Create a temporary table to weed out duplicate rowid combinations. The
-    table has a single column that is a concatenation of all rowids in the
-    combination. 
-
-    Depending on the needed length, there are two cases:
-
-    1. When the length of the column < max_key_length:
-
-      CREATE TABLE tmp (col VARBINARY(n) NOT NULL, UNIQUE KEY(col));
-
-    2. Otherwise (not a valid SQL syntax but internally supported):
-
-      CREATE TABLE tmp (col VARBINARY NOT NULL, UNIQUE CONSTRAINT(col));
-
-    The code in this function was produced by extraction of relevant parts
-    from create_tmp_table().
-
-  RETURN
-    created table
-    NULL on error
-*/
-
-TABLE *create_duplicate_weedout_tmp_table(THD *thd, 
-                                          uint uniq_tuple_length_arg,
-                                          SJ_TMP_TABLE *sjtbl)
-{
-  MEM_ROOT *mem_root_save, own_root;
-  TABLE *table;
-  TABLE_SHARE *share;
-  uint  temp_pool_slot=MY_BIT_NONE;
-  char	*tmpname,path[FN_REFLEN];
-  Field **reg_field;
-  KEY_PART_INFO *key_part_info;
-  KEY *keyinfo;
-  uchar *group_buff;
-  uchar *bitmaps;
-  uint *blob_field;
-  MI_COLUMNDEF *recinfo, *start_recinfo;
-  bool using_unique_constraint=false;
-  Field *field, *key_field;
-  uint blob_count, null_pack_length, null_count;
-  uchar *null_flags;
-  uchar *pos;
-  
-  /*
-    STEP 1: Get temporary table name
-  */
-  statistic_increment(thd->status_var.created_tmp_tables, &LOCK_status);
-  if (use_temp_pool && !(test_flags & TEST_KEEP_TMP_TABLES))
-    temp_pool_slot = bitmap_lock_set_next(&temp_pool);
-
-  if (temp_pool_slot != MY_BIT_NONE) // we got a slot
-    sprintf(path, "%s_%lx_%i", tmp_file_prefix,
-	    current_pid, temp_pool_slot);
-  else
-  {
-    /* if we run out of slots or we are not using tempool */
-    sprintf(path,"%s%lx_%lx_%x", tmp_file_prefix,current_pid,
-            thd->thread_id, thd->tmp_table++);
-  }
-  fn_format(path, path, mysql_tmpdir, "", MY_REPLACE_EXT|MY_UNPACK_FILENAME);
-
-  /* STEP 2: Figure if we'll be using a key or blob+constraint */
-  if (uniq_tuple_length_arg >= CONVERT_IF_BIGGER_TO_BLOB)
-    using_unique_constraint= true;
-
-  /* STEP 3: Allocate memory for temptable description */
-  init_sql_alloc(&own_root, TABLE_ALLOC_BLOCK_SIZE, 0);
-  if (!multi_alloc_root(&own_root,
-                        &table, sizeof(*table),
-                        &share, sizeof(*share),
-                        &reg_field, sizeof(Field*) * (1+1),
-                        &blob_field, sizeof(uint)*2,
-                        &keyinfo, sizeof(*keyinfo),
-                        &key_part_info, sizeof(*key_part_info) * 2,
-                        &start_recinfo,
-                        sizeof(*recinfo)*(1*2+4),
-                        &tmpname, (uint) strlen(path)+1,
-                        &group_buff, (!using_unique_constraint ?
-                                      uniq_tuple_length_arg : 0),
-                        &bitmaps, bitmap_buffer_size(1)*2,
-                        NullS))
-  {
-    if (temp_pool_slot != MY_BIT_NONE)
-      bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
-    return(NULL);
-  }
-  stpcpy(tmpname,path);
-  
-
-  /* STEP 4: Create TABLE description */
-  memset(table, 0, sizeof(*table));
-  memset(reg_field, 0, sizeof(Field*)*2);
-
-  table->mem_root= own_root;
-  mem_root_save= thd->mem_root;
-  thd->mem_root= &table->mem_root;
-
-  table->field=reg_field;
-  table->alias= "weedout-tmp";
-  table->reginfo.lock_type=TL_WRITE;	/* Will be updated */
-  table->db_stat=HA_OPEN_KEYFILE+HA_OPEN_RNDFILE;
-  table->map=1;
-  table->temp_pool_slot = temp_pool_slot;
-  table->copy_blobs= 1;
-  table->in_use= thd;
-  table->quick_keys.init();
-  table->covering_keys.init();
-  table->keys_in_use_for_query.init();
-
-  table->s= share;
-  init_tmp_table_share(thd, share, "", 0, tmpname, tmpname);
-  share->blob_field= blob_field;
-  share->blob_ptr_size= portable_sizeof_char_ptr;
-  share->db_low_byte_first=1;                // True for HEAP and MyISAM
-  share->table_charset= NULL;
-  share->primary_key= MAX_KEY;               // Indicate no primary key
-  share->keys_for_keyread.init();
-  share->keys_in_use.init();
-
-  blob_count= 0;
-
-  /* Create the field */
-  {
-    /*
-      For the sake of uniformity, always use Field_varstring.
-    */
-    field= new Field_varstring(uniq_tuple_length_arg, false, "rowids", share,
-                               &my_charset_bin);
-    if (!field)
-      return(0);
-    field->table= table;
-    field->key_start.init(0);
-    field->part_of_key.init(0);
-    field->part_of_sortkey.init(0);
-    field->unireg_check= Field::NONE;
-    field->flags= (NOT_NULL_FLAG | BINARY_FLAG | NO_DEFAULT_VALUE_FLAG);
-    field->reset_fields();
-    field->init(table);
-    field->orig_table= NULL;
-     
-    field->field_index= 0;
-    
-    *(reg_field++)= field;
-    *blob_field= 0;
-    *reg_field= 0;
-
-    share->fields= 1;
-    share->blob_fields= 0;
-  }
-
-  uint reclength= field->pack_length();
-  if (using_unique_constraint)
-  { 
-    share->db_plugin= ha_lock_engine(0, myisam_hton);
-    table->file= get_new_handler(share, &table->mem_root,
-                                 share->db_type());
-    assert(uniq_tuple_length_arg <= table->file->max_key_length());
-  }
-  else
-  {
-    share->db_plugin= ha_lock_engine(0, heap_hton);
-    table->file= get_new_handler(share, &table->mem_root,
-                                 share->db_type());
-  }
-  if (!table->file)
-    goto err;
-
-  null_count=1;
-  
-  null_pack_length= 1;
-  reclength += null_pack_length;
-
-  share->reclength= reclength;
-  {
-    uint alloc_length=ALIGN_SIZE(share->reclength + MI_UNIQUE_HASH_LENGTH+1);
-    share->rec_buff_length= alloc_length;
-    if (!(table->record[0]= (uchar*)
-                            alloc_root(&table->mem_root, alloc_length*3)))
-      goto err;
-    table->record[1]= table->record[0]+alloc_length;
-    share->default_values= table->record[1]+alloc_length;
-  }
-  setup_tmp_table_column_bitmaps(table, bitmaps);
-
-  recinfo= start_recinfo;
-  null_flags=(uchar*) table->record[0];
-  pos=table->record[0]+ null_pack_length;
-  if (null_pack_length)
-  {
-    memset(recinfo, 0, sizeof(*recinfo));
-    recinfo->type=FIELD_NORMAL;
-    recinfo->length=null_pack_length;
-    recinfo++;
-    memset(null_flags, 255, null_pack_length);	// Set null fields
-
-    table->null_flags= (uchar*) table->record[0];
-    share->null_fields= null_count;
-    share->null_bytes= null_pack_length;
-  }
-  null_count=1;
-
-  {
-    //Field *field= *reg_field;
-    uint length;
-    memset(recinfo, 0, sizeof(*recinfo));
-    field->move_field(pos,(uchar*) 0,0);
-
-    field->reset();
-    /*
-      Test if there is a default field value. The test for ->ptr is to skip
-      'offset' fields generated by initalize_tables
-    */
-    // Initialize the table field:
-    memset(field->ptr, 0, field->pack_length());
-
-    length=field->pack_length();
-    pos+= length;
-
-    /* Make entry for create table */
-    recinfo->length=length;
-    if (field->flags & BLOB_FLAG)
-      recinfo->type= (int) FIELD_BLOB;
-    else
-      recinfo->type=FIELD_NORMAL;
-
-    field->table_name= &table->alias;
-  }
-
-  //param->recinfo=recinfo;
-  //store_record(table,s->default_values);        // Make empty default record
-
-  if (thd->variables.tmp_table_size == ~ (uint64_t) 0)		// No limit
-    share->max_rows= ~(ha_rows) 0;
-  else
-    share->max_rows= (ha_rows) (((share->db_type() == heap_hton) ?
-                                 min(thd->variables.tmp_table_size,
-                                     thd->variables.max_heap_table_size) :
-                                 thd->variables.tmp_table_size) /
-			         share->reclength);
-  set_if_bigger(share->max_rows,1);		// For dummy start options
-
-
-  //// keyinfo= param->keyinfo;
-  if (true)
-  {
-    share->keys=1;
-    share->uniques= test(using_unique_constraint);
-    table->key_info=keyinfo;
-    keyinfo->key_part=key_part_info;
-    keyinfo->flags=HA_NOSAME;
-    keyinfo->usable_key_parts= keyinfo->key_parts= 1;
-    keyinfo->key_length=0;
-    keyinfo->rec_per_key=0;
-    keyinfo->algorithm= HA_KEY_ALG_UNDEF;
-    keyinfo->name= (char*) "weedout_key";
-    {
-      key_part_info->null_bit=0;
-      key_part_info->field=  field;
-      key_part_info->offset= field->offset(table->record[0]);
-      key_part_info->length= (uint16_t) field->key_length();
-      key_part_info->type=   (uint8_t) field->key_type();
-      key_part_info->key_type = FIELDFLAG_BINARY;
-      if (!using_unique_constraint)
-      {
-	if (!(key_field= field->new_key_field(thd->mem_root, table,
-                                              group_buff,
-                                              field->null_ptr,
-                                              field->null_bit)))
-	  goto err;
-        key_part_info->key_part_flag|= HA_END_SPACE_ARE_EQUAL; //todo need this?
-      }
-      keyinfo->key_length+=  key_part_info->length;
-    }
-  }
-
-  if (thd->is_fatal_error)				// If end of memory
-    goto err;
-  share->db_record_offset= 1;
-  if (share->db_type() == myisam_hton)
-  {
-    recinfo++;
-    if (create_myisam_tmp_table(table, keyinfo, start_recinfo, &recinfo, 0))
-      goto err;
-  }
-  sjtbl->start_recinfo= start_recinfo;
-  sjtbl->recinfo=       recinfo;
-  if (open_tmp_table(table))
-    goto err;
-
-  thd->mem_root= mem_root_save;
-  return(table);
-
-err:
-  thd->mem_root= mem_root_save;
-  free_tmp_table(thd,table);                    /* purecov: inspected */
-  if (temp_pool_slot != MY_BIT_NONE)
-    bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
-  return(NULL);				/* purecov: inspected */
-}
-
-/****************************************************************************/
-
-/**
-  Create a reduced TABLE object with properly set up Field list from a
-  list of field definitions.
-
-    The created table doesn't have a table handler associated with
-    it, has no keys, no group/distinct, no copy_funcs array.
-    The sole purpose of this TABLE object is to use the power of Field
-    class to read/write data to/from table->record[0]. Then one can store
-    the record in any container (RB tree, hash, etc).
-    The table is created in THD mem_root, so are the table's fields.
-    Consequently, if you don't BLOB fields, you don't need to free it.
-
-  @param thd         connection handle
-  @param field_list  list of column definitions
-
-  @return
-    0 if out of memory, TABLE object in case of success
-*/
-
-TABLE *create_virtual_tmp_table(THD *thd, List<Create_field> &field_list)
-{
-  uint field_count= field_list.elements;
-  uint blob_count= 0;
-  Field **field;
-  Create_field *cdef;                           /* column definition */
-  uint record_length= 0;
-  uint null_count= 0;                 /* number of columns which may be null */
-  uint null_pack_length;              /* NULL representation array length */
-  uint *blob_field;
-  uchar *bitmaps;
-  TABLE *table;
-  TABLE_SHARE *share;
-
-  if (!multi_alloc_root(thd->mem_root,
-                        &table, sizeof(*table),
-                        &share, sizeof(*share),
-                        &field, (field_count + 1) * sizeof(Field*),
-                        &blob_field, (field_count+1) *sizeof(uint),
-                        &bitmaps, bitmap_buffer_size(field_count)*2,
-                        NullS))
-    return 0;
-
-  memset(table, 0, sizeof(*table));
-  memset(share, 0, sizeof(*share));
-  table->field= field;
-  table->s= share;
-  share->blob_field= blob_field;
-  share->fields= field_count;
-  share->blob_ptr_size= portable_sizeof_char_ptr;
-  setup_tmp_table_column_bitmaps(table, bitmaps);
-
-  /* Create all fields and calculate the total length of record */
-  List_iterator_fast<Create_field> it(field_list);
-  while ((cdef= it++))
-  {
-    *field= make_field(share, 0, cdef->length,
-                       (uchar*) (f_maybe_null(cdef->pack_flag) ? "" : 0),
-                       f_maybe_null(cdef->pack_flag) ? 1 : 0,
-                       cdef->pack_flag, cdef->sql_type, cdef->charset,
-                       cdef->unireg_check,
-                       cdef->interval, cdef->field_name);
-    if (!*field)
-      goto error;
-    (*field)->init(table);
-    record_length+= (*field)->pack_length();
-    if (! ((*field)->flags & NOT_NULL_FLAG))
-      null_count++;
-
-    if ((*field)->flags & BLOB_FLAG)
-      share->blob_field[blob_count++]= (uint) (field - table->field);
-
-    field++;
-  }
-  *field= NULL;                             /* mark the end of the list */
-  share->blob_field[blob_count]= 0;            /* mark the end of the list */
-  share->blob_fields= blob_count;
-
-  null_pack_length= (null_count + 7)/8;
-  share->reclength= record_length + null_pack_length;
-  share->rec_buff_length= ALIGN_SIZE(share->reclength + 1);
-  table->record[0]= (uchar*) thd->alloc(share->rec_buff_length);
-  if (!table->record[0])
-    goto error;
-
-  if (null_pack_length)
-  {
-    table->null_flags= (uchar*) table->record[0];
-    share->null_fields= null_count;
-    share->null_bytes= null_pack_length;
-  }
-
-  table->in_use= thd;           /* field->reset() may access table->in_use */
-  {
-    /* Set up field pointers */
-    uchar *null_pos= table->record[0];
-    uchar *field_pos= null_pos + share->null_bytes;
-    uint null_bit= 1;
-
-    for (field= table->field; *field; ++field)
-    {
-      Field *cur_field= *field;
-      if ((cur_field->flags & NOT_NULL_FLAG))
-        cur_field->move_field(field_pos);
-      else
-      {
-        cur_field->move_field(field_pos, (uchar*) null_pos, null_bit);
-        null_bit<<= 1;
-        if (null_bit == (1 << 8))
-        {
-          ++null_pos;
-          null_bit= 1;
-        }
-      }
-      cur_field->reset();
-
-      field_pos+= cur_field->pack_length();
-    }
-  }
-  return table;
-error:
-  for (field= table->field; *field; ++field)
-    delete *field;                         /* just invokes field destructor */
-  return 0;
-}
-
-
-static bool open_tmp_table(TABLE *table)
-{
-  int error;
-  if ((error=table->file->ha_open(table, table->s->table_name.str,O_RDWR,
-                                  HA_OPEN_TMP_TABLE | HA_OPEN_INTERNAL_TABLE)))
-  {
-    table->file->print_error(error,MYF(0)); /* purecov: inspected */
-    table->db_stat=0;
-    return(1);
-  }
-  (void) table->file->extra(HA_EXTRA_QUICK);		/* Faster */
-  return(0);
-}
-
-
-/*
-  Create MyISAM temporary table
-
-  SYNOPSIS
-    create_myisam_tmp_table()
-      table           Table object that descrimes the table to be created
-      keyinfo         Description of the index (there is always one index)
-      start_recinfo   MyISAM's column descriptions
-      recinfo INOUT   End of MyISAM's column descriptions
-      options         Option bits
-   
-  DESCRIPTION
-    Create a MyISAM temporary table according to passed description. The is
-    assumed to have one unique index or constraint.
-
-    The passed array or MI_COLUMNDEF structures must have this form:
-
-      1. 1-byte column (afaiu for 'deleted' flag) (note maybe not 1-byte
-         when there are many nullable columns)
-      2. Table columns
-      3. One free MI_COLUMNDEF element (*recinfo points here)
-   
-    This function may use the free element to create hash column for unique
-    constraint.
-
-   RETURN
-     false - OK
-     true  - Error
-*/
-
-static bool create_myisam_tmp_table(TABLE *table, KEY *keyinfo, 
-                                    MI_COLUMNDEF *start_recinfo,
-                                    MI_COLUMNDEF **recinfo, 
-				    uint64_t options)
-{
-  int error;
-  MI_KEYDEF keydef;
-  MI_UNIQUEDEF uniquedef;
-  TABLE_SHARE *share= table->s;
-
-  if (share->keys)
-  {						// Get keys for ni_create
-    bool using_unique_constraint=0;
-    HA_KEYSEG *seg= (HA_KEYSEG*) alloc_root(&table->mem_root,
-                                            sizeof(*seg) * keyinfo->key_parts);
-    if (!seg)
-      goto err;
-
-    memset(seg, 0, sizeof(*seg) * keyinfo->key_parts);
-    if (keyinfo->key_length >= table->file->max_key_length() ||
-	keyinfo->key_parts > table->file->max_key_parts() ||
-	share->uniques)
-    {
-      /* Can't create a key; Make a unique constraint instead of a key */
-      share->keys=    0;
-      share->uniques= 1;
-      using_unique_constraint=1;
-      memset(&uniquedef, 0, sizeof(uniquedef));
-      uniquedef.keysegs=keyinfo->key_parts;
-      uniquedef.seg=seg;
-      uniquedef.null_are_equal=1;
-
-      /* Create extra column for hash value */
-      memset(*recinfo, 0, sizeof(**recinfo));
-      (*recinfo)->type= FIELD_CHECK;
-      (*recinfo)->length=MI_UNIQUE_HASH_LENGTH;
-      (*recinfo)++;
-      share->reclength+=MI_UNIQUE_HASH_LENGTH;
-    }
-    else
-    {
-      /* Create an unique key */
-      memset(&keydef, 0, sizeof(keydef));
-      keydef.flag=HA_NOSAME | HA_BINARY_PACK_KEY | HA_PACK_KEY;
-      keydef.keysegs=  keyinfo->key_parts;
-      keydef.seg= seg;
-    }
-    for (uint i=0; i < keyinfo->key_parts ; i++,seg++)
-    {
-      Field *field=keyinfo->key_part[i].field;
-      seg->flag=     0;
-      seg->language= field->charset()->number;
-      seg->length=   keyinfo->key_part[i].length;
-      seg->start=    keyinfo->key_part[i].offset;
-      if (field->flags & BLOB_FLAG)
-      {
-	seg->type=
-	((keyinfo->key_part[i].key_type & FIELDFLAG_BINARY) ?
-	 HA_KEYTYPE_VARBINARY2 : HA_KEYTYPE_VARTEXT2);
-	seg->bit_start= (uint8_t)(field->pack_length() - share->blob_ptr_size);
-	seg->flag= HA_BLOB_PART;
-	seg->length=0;			// Whole blob in unique constraint
-      }
-      else
-      {
-	seg->type= keyinfo->key_part[i].type;
-      }
-      if (!(field->flags & NOT_NULL_FLAG))
-      {
-	seg->null_bit= field->null_bit;
-	seg->null_pos= (uint) (field->null_ptr - (uchar*) table->record[0]);
-	/*
-	  We are using a GROUP BY on something that contains NULL
-	  In this case we have to tell MyISAM that two NULL should
-	  on INSERT be regarded at the same value
-	*/
-	if (!using_unique_constraint)
-	  keydef.flag|= HA_NULL_ARE_EQUAL;
-      }
-    }
-  }
-  MI_CREATE_INFO create_info;
-  memset(&create_info, 0, sizeof(create_info));
-
-  if ((options & (OPTION_BIG_TABLES | SELECT_SMALL_RESULT)) ==
-      OPTION_BIG_TABLES)
-    create_info.data_file_length= ~(uint64_t) 0;
-
-  if ((error=mi_create(share->table_name.str, share->keys, &keydef,
-		       (uint) (*recinfo-start_recinfo),
-		       start_recinfo,
-		       share->uniques, &uniquedef,
-		       &create_info,
-		       HA_CREATE_TMP_TABLE)))
-  {
-    table->file->print_error(error,MYF(0));	/* purecov: inspected */
-    table->db_stat=0;
-    goto err;
-  }
-  status_var_increment(table->in_use->status_var.created_tmp_disk_tables);
-  share->db_record_offset= 1;
-  return(0);
- err:
-  return(1);
-}
-
-
-void
-free_tmp_table(THD *thd, TABLE *entry)
-{
-  MEM_ROOT own_root= entry->mem_root;
-  const char *save_proc_info;
-
-  save_proc_info=thd->proc_info;
-  thd_proc_info(thd, "removing tmp table");
-
-  if (entry->file)
-  {
-    if (entry->db_stat)
-      entry->file->ha_drop_table(entry->s->table_name.str);
-    else
-      entry->file->ha_delete_table(entry->s->table_name.str);
-    delete entry->file;
-  }
-
-  /* free blobs */
-  for (Field **ptr=entry->field ; *ptr ; ptr++)
-    (*ptr)->free();
-  free_io_cache(entry);
-
-  if (entry->temp_pool_slot != MY_BIT_NONE)
-    bitmap_lock_clear_bit(&temp_pool, entry->temp_pool_slot);
-
-  plugin_unlock(0, entry->s->db_plugin);
-
-  free_root(&own_root, MYF(0)); /* the table is allocated in its own root */
-  thd_proc_info(thd, save_proc_info);
-
-  return;
-}
-
-/**
-  If a HEAP table gets full, create a MyISAM table and copy all rows
-  to this.
-*/
-
-bool create_myisam_from_heap(THD *thd, TABLE *table,
-                             MI_COLUMNDEF *start_recinfo,
-                             MI_COLUMNDEF **recinfo, 
-			     int error, bool ignore_last_dupp_key_error)
-{
-  TABLE new_table;
-  TABLE_SHARE share;
-  const char *save_proc_info;
-  int write_err;
-
-  if (table->s->db_type() != heap_hton || 
-      error != HA_ERR_RECORD_FILE_FULL)
-  {
-    table->file->print_error(error,MYF(0));
-    return(1);
-  }
-  new_table= *table;
-  share= *table->s;
-  new_table.s= &share;
-  new_table.s->db_plugin= ha_lock_engine(thd, myisam_hton);
-  if (!(new_table.file= get_new_handler(&share, &new_table.mem_root,
-                                        new_table.s->db_type())))
-    return(1);				// End of memory
-
-  save_proc_info=thd->proc_info;
-  thd_proc_info(thd, "converting HEAP to MyISAM");
-
-  if (create_myisam_tmp_table(&new_table, table->key_info, start_recinfo,
-                              recinfo, thd->lex->select_lex.options | 
-                                               thd->options))
-    goto err2;
-  if (open_tmp_table(&new_table))
-    goto err1;
-  if (table->file->indexes_are_disabled())
-    new_table.file->ha_disable_indexes(HA_KEY_SWITCH_ALL);
-  table->file->ha_index_or_rnd_end();
-  table->file->ha_rnd_init(1);
-  if (table->no_rows)
-  {
-    new_table.file->extra(HA_EXTRA_NO_ROWS);
-    new_table.no_rows=1;
-  }
-
-#ifdef TO_BE_DONE_LATER_IN_4_1
-  /*
-    To use start_bulk_insert() (which is new in 4.1) we need to find
-    all places where a corresponding end_bulk_insert() should be put.
-  */
-  table->file->info(HA_STATUS_VARIABLE); /* update table->file->stats.records */
-  new_table.file->ha_start_bulk_insert(table->file->stats.records);
-#else
-  /* HA_EXTRA_WRITE_CACHE can stay until close, no need to disable it */
-  new_table.file->extra(HA_EXTRA_WRITE_CACHE);
-#endif
-
-  /*
-    copy all old rows from heap table to MyISAM table
-    This is the only code that uses record[1] to read/write but this
-    is safe as this is a temporary MyISAM table without timestamp/autoincrement.
-  */
-  while (!table->file->rnd_next(new_table.record[1]))
-  {
-    write_err= new_table.file->ha_write_row(new_table.record[1]);
-    if (write_err)
-      goto err;
-  }
-  /* copy row that filled HEAP table */
-  if ((write_err=new_table.file->ha_write_row(table->record[0])))
-  {
-    if (new_table.file->is_fatal_error(write_err, HA_CHECK_DUP) ||
-	!ignore_last_dupp_key_error)
-      goto err;
-  }
-
-  /* remove heap table and change to use myisam table */
-  (void) table->file->ha_rnd_end();
-  (void) table->file->close();                  // This deletes the table !
-  delete table->file;
-  table->file=0;
-  plugin_unlock(0, table->s->db_plugin);
-  share.db_plugin= my_plugin_lock(0, &share.db_plugin);
-  new_table.s= table->s;                       // Keep old share
-  *table= new_table;
-  *table->s= share;
-  
-  table->file->change_table_ptr(table, table->s);
-  table->use_all_columns();
-  if (save_proc_info)
-  {
-    const char *new_proc_info=
-      (!strcmp(save_proc_info,"Copying to tmp table") ?
-      "Copying to tmp table on disk" : save_proc_info);
-    thd_proc_info(thd, new_proc_info);
-  }
-  return(0);
-
- err:
-  table->file->print_error(write_err, MYF(0));
-  (void) table->file->ha_rnd_end();
-  (void) new_table.file->close();
- err1:
-  new_table.file->ha_delete_table(new_table.s->table_name.str);
- err2:
-  delete new_table.file;
-  thd_proc_info(thd, save_proc_info);
-  table->mem_root= new_table.mem_root;
-  return(1);
-}
-
 
 /**
   @details

=== modified file 'drizzled/sql_select.h'
--- drizzled/sql_select.h	2008-08-12 21:13:01 +0000
+++ drizzled/sql_select.h	2008-08-15 21:14:46 +0000
@@ -366,48 +366,6 @@
 } ROLLUP;
 
 
-/*
-  Describes use of one temporary table to weed out join duplicates.
-  The temporar
-
-  Used to
-    - create a temp table
-    - when we reach the weed-out tab, walk through rowid-ed tabs and
-      and copy rowids.
-      For each table we need
-       - rowid offset
-       - null bit address.
-*/
-
-class SJ_TMP_TABLE : public Sql_alloc
-{
-public:
-  /* Array of pointers to tables that should be "used" */
-  class TAB
-  {
-  public:
-    JOIN_TAB *join_tab;
-    uint rowid_offset;
-    ushort null_byte;
-    uchar null_bit;
-  };
-  TAB *tabs;
-  TAB *tabs_end;
-
-  uint null_bits;
-  uint null_bytes;
-  uint rowid_len;
-
-  TABLE *tmp_table;
-
-  MI_COLUMNDEF *start_recinfo;
-  MI_COLUMNDEF *recinfo;
-
-  /* Pointer to next table (next->start_idx > this->end_idx) */
-  SJ_TMP_TABLE *next; 
-};
-
-
 class JOIN :public Sql_alloc
 {
   JOIN(const JOIN &rhs);                        /**< not implemented */
@@ -703,10 +661,6 @@
 		       uint elements, List<Item> &fields);
 void copy_fields(TMP_TABLE_PARAM *param);
 void copy_funcs(Item **func_ptr);
-bool create_myisam_from_heap(THD *thd, TABLE *table,
-                             MI_COLUMNDEF *start_recinfo,
-                             MI_COLUMNDEF **recinfo, 
-			     int error, bool ignore_last_dupp_key_error);
 uint find_shortest_key(TABLE *table, const key_map *usable_keys);
 Field* create_tmp_field_from_field(THD *thd, Field* org_field,
                                    const char *name, TABLE *table,

=== modified file 'drizzled/sql_union.cc'
--- drizzled/sql_union.cc	2008-08-06 18:46:02 +0000
+++ drizzled/sql_union.cc	2008-08-15 21:14:46 +0000
@@ -612,7 +612,7 @@
     delete union_result;
     union_result=0; // Safety
     if (table)
-      free_tmp_table(thd, table);
+      table->free_tmp_table(thd);
     table= 0; // Safety
   }
 

=== modified file 'drizzled/sql_update.cc'
--- drizzled/sql_update.cc	2008-08-12 21:13:01 +0000
+++ drizzled/sql_update.cc	2008-08-15 21:14:46 +0000
@@ -1330,7 +1330,7 @@
     {
       if (tmp_tables[cnt])
       {
-	free_tmp_table(thd, tmp_tables[cnt]);
+	tmp_tables[cnt]->free_tmp_table(thd);
 	tmp_table_param[cnt].cleanup();
       }
     }

=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2008-08-15 15:01:59 +0000
+++ drizzled/table.cc	2008-08-15 21:14:46 +0000
@@ -19,12 +19,12 @@
 #include <drizzled/server_includes.h>
 #include <drizzled/drizzled_error_messages.h>
 
+#include "tmp_table.h"
+#include "sj_tmp_table.h"
+
 /* INFORMATION_SCHEMA name */
 LEX_STRING INFORMATION_SCHEMA_NAME= {C_STRING_WITH_LEN("information_schema")};
 
-/* DRIZZLE_SCHEMA name */
-LEX_STRING DRIZZLE_SCHEMA_NAME= {C_STRING_WITH_LEN("mysql")};
-
 /* Functions defined in this file */
 
 void open_table_error(TABLE_SHARE *share, int error, int db_errno,
@@ -2019,20 +2019,41 @@
   return (file);
 } /* create_frm */
 
-
-void update_create_info_from_table(HA_CREATE_INFO *create_info, TABLE *table)
-{
-  TABLE_SHARE *share= table->getShare();
-
-  create_info->max_rows= share->max_rows;
-  create_info->min_rows= share->min_rows;
-  create_info->table_options= share->db_create_options;
-  create_info->avg_row_length= share->avg_row_length;
-  create_info->block_size= share->block_size;
-  create_info->row_type= share->row_type;
-  create_info->default_table_charset= share->table_charset;
+/*
+  Set up column usage bitmaps for a temporary table
+
+  IMPLEMENTATION
+    For temporary tables, we need one bitmap with all columns set and
+    a tmp_set bitmap to be used by things like filesort.
+*/
+
+void Table::setup_tmp_table_column_bitmaps(uchar *bitmaps)
+{
+  uint field_count= s->fields;
+
+  bitmap_init(&this->def_read_set, (my_bitmap_map*) bitmaps, field_count, false);
+  bitmap_init(&this->tmp_set, (my_bitmap_map*) (bitmaps+ bitmap_buffer_size(field_count)), field_count, false);
+
+  /* write_set and all_set are copies of read_set */
+  def_write_set= def_read_set;
+  s->all_set= def_read_set;
+  bitmap_set_all(&this->s->all_set);
+  default_column_bitmaps();
+}
+
+
+
+void Table::updateCreateInfo(HA_CREATE_INFO *create_info)
+{
+  create_info->max_rows= s->max_rows;
+  create_info->min_rows= s->min_rows;
+  create_info->table_options= s->db_create_options;
+  create_info->avg_row_length= s->avg_row_length;
+  create_info->block_size= s->block_size;
+  create_info->row_type= s->row_type;
+  create_info->default_table_charset= s->table_charset;
   create_info->table_charset= 0;
-  create_info->comment= share->comment;
+  create_info->comment= s->comment;
 
   return;
 }
@@ -3515,10 +3536,9 @@
 
 size_t max_row_length(TABLE *table, const uchar *data)
 {
-  TABLE_SHARE *table_s= table->s;
-  size_t length= table_s->reclength + 2 * table_s->fields;
-  uint *const beg= table_s->blob_field;
-  uint *const end= beg + table_s->blob_fields;
+  size_t length= table->getRecordLength() + 2 * table->sizeFields();
+  uint *const beg= table->getBlobField();
+  uint *const end= beg + table->sizeBlobFields();
 
   for (uint *ptr= beg ; ptr != end ; ++ptr)
   {
@@ -3573,6 +3593,1498 @@
   return(FRMTYPE_TABLE);                   // Is probably a .frm table
 }
 
+/****************************************************************************
+ Functions for creating temporary tables.
+****************************************************************************/
+
+
+/* Prototypes */
+void free_tmp_table(THD *thd, TABLE *entry);
+
+/**
+  Create field for temporary table from given field.
+
+  @param thd	       Thread handler
+  @param org_field    field from which new field will be created
+  @param name         New field name
+  @param table	       Temporary table
+  @param item	       !=NULL if item->result_field should point to new field.
+                      This is relevant for how fill_record() is going to work:
+                      If item != NULL then fill_record() will update
+                      the record in the original table.
+                      If item == NULL then fill_record() will update
+                      the temporary table
+  @param convert_blob_length   If >0 create a varstring(convert_blob_length)
+                               field instead of blob.
+
+  @retval
+    NULL		on error
+  @retval
+    new_created field
+*/
+
+Field *create_tmp_field_from_field(THD *thd, Field *org_field,
+                                   const char *name, TABLE *table,
+                                   Item_field *item, uint convert_blob_length)
+{
+  Field *new_field;
+
+  /* 
+    Make sure that the blob fits into a Field_varstring which has 
+    2-byte lenght. 
+  */
+  if (convert_blob_length && convert_blob_length <= Field_varstring::MAX_SIZE &&
+      (org_field->flags & BLOB_FLAG))
+    new_field= new Field_varstring(convert_blob_length,
+                                   org_field->maybe_null(),
+                                   org_field->field_name, table->s,
+                                   org_field->charset());
+  else
+    new_field= org_field->new_field(thd->mem_root, table,
+                                    table == org_field->table);
+  if (new_field)
+  {
+    new_field->init(table);
+    new_field->orig_table= org_field->orig_table;
+    if (item)
+      item->result_field= new_field;
+    else
+      new_field->field_name= name;
+    new_field->flags|= (org_field->flags & NO_DEFAULT_VALUE_FLAG);
+    if (org_field->maybe_null() || (item && item->maybe_null))
+      new_field->flags&= ~NOT_NULL_FLAG;	// Because of outer join
+    if (org_field->type() == DRIZZLE_TYPE_VARCHAR)
+      table->s->db_create_options|= HA_OPTION_PACK_RECORD;
+    else if (org_field->type() == DRIZZLE_TYPE_DOUBLE)
+      ((Field_double *) new_field)->not_fixed= true;
+  }
+  return new_field;
+}
+
+/**
+  Create field for temporary table using type of given item.
+
+  @param thd                   Thread handler
+  @param item                  Item to create a field for
+  @param table                 Temporary table
+  @param copy_func             If set and item is a function, store copy of
+                               item in this array
+  @param modify_item           1 if item->result_field should point to new
+                               item. This is relevent for how fill_record()
+                               is going to work:
+                               If modify_item is 1 then fill_record() will
+                               update the record in the original table.
+                               If modify_item is 0 then fill_record() will
+                               update the temporary table
+  @param convert_blob_length   If >0 create a varstring(convert_blob_length)
+                               field instead of blob.
+
+  @retval
+    0  on error
+  @retval
+    new_created field
+*/
+
+static Field *create_tmp_field_from_item(THD *thd __attribute__((unused)),
+                                         Item *item, TABLE *table,
+                                         Item ***copy_func, bool modify_item,
+                                         uint convert_blob_length)
+{
+  bool maybe_null= item->maybe_null;
+  Field *new_field;
+
+  switch (item->result_type()) {
+  case REAL_RESULT:
+    new_field= new Field_double(item->max_length, maybe_null,
+                                item->name, item->decimals, true);
+    break;
+  case INT_RESULT:
+    /* 
+      Select an integer type with the minimal fit precision.
+      MY_INT32_NUM_DECIMAL_DIGITS is sign inclusive, don't consider the sign.
+      Values with MY_INT32_NUM_DECIMAL_DIGITS digits may or may not fit into 
+      Field_long : make them Field_int64_t.  
+    */
+    if (item->max_length >= (MY_INT32_NUM_DECIMAL_DIGITS - 1))
+      new_field=new Field_int64_t(item->max_length, maybe_null,
+                                   item->name, item->unsigned_flag);
+    else
+      new_field=new Field_long(item->max_length, maybe_null,
+                               item->name, item->unsigned_flag);
+    break;
+  case STRING_RESULT:
+    assert(item->collation.collation);
+  
+    enum enum_field_types type;
+    /*
+      DATE/TIME fields have STRING_RESULT result type. 
+      To preserve type they needed to be handled separately.
+    */
+    if ((type= item->field_type()) == DRIZZLE_TYPE_DATETIME ||
+        type == DRIZZLE_TYPE_TIME || type == DRIZZLE_TYPE_NEWDATE ||
+        type == DRIZZLE_TYPE_TIMESTAMP)
+      new_field= item->tmp_table_field_from_field_type(table, 1);
+    /* 
+      Make sure that the blob fits into a Field_varstring which has 
+      2-byte lenght. 
+    */
+    else if (item->max_length/item->collation.collation->mbmaxlen > 255 &&
+             convert_blob_length <= Field_varstring::MAX_SIZE && 
+             convert_blob_length)
+      new_field= new Field_varstring(convert_blob_length, maybe_null,
+                                     item->name, table->s,
+                                     item->collation.collation);
+    else
+      new_field= item->make_string_field(table);
+    new_field->set_derivation(item->collation.derivation);
+    break;
+  case DECIMAL_RESULT:
+  {
+    uint8_t dec= item->decimals;
+    uint8_t intg= ((Item_decimal *) item)->decimal_precision() - dec;
+    uint32_t len= item->max_length;
+
+    /*
+      Trying to put too many digits overall in a DECIMAL(prec,dec)
+      will always throw a warning. We must limit dec to
+      DECIMAL_MAX_SCALE however to prevent an assert() later.
+    */
+
+    if (dec > 0)
+    {
+      signed int overflow;
+
+      dec= min(dec, (uint8_t)DECIMAL_MAX_SCALE);
+
+      /*
+        If the value still overflows the field with the corrected dec,
+        we'll throw out decimals rather than integers. This is still
+        bad and of course throws a truncation warning.
+        +1: for decimal point
+      */
+
+      overflow= my_decimal_precision_to_length(intg + dec, dec,
+                                               item->unsigned_flag) - len;
+
+      if (overflow > 0)
+        dec= max(0, dec - overflow);            // too long, discard fract
+      else
+        len -= item->decimals - dec;            // corrected value fits
+    }
+
+    new_field= new Field_new_decimal(len, maybe_null, item->name,
+                                     dec, item->unsigned_flag);
+    break;
+  }
+  case ROW_RESULT:
+  default:
+    // This case should never be choosen
+    assert(0);
+    new_field= 0;
+    break;
+  }
+  if (new_field)
+    new_field->init(table);
+    
+  if (copy_func && item->is_result_field())
+    *((*copy_func)++) = item;			// Save for copy_funcs
+  if (modify_item)
+    item->set_result_field(new_field);
+  if (item->type() == Item::NULL_ITEM)
+    new_field->is_created_from_null_item= true;
+  return new_field;
+}
+
+
+/**
+  Create field for information schema table.
+
+  @param thd		Thread handler
+  @param table		Temporary table
+  @param item		Item to create a field for
+
+  @retval
+    0			on error
+  @retval
+    new_created field
+*/
+
+Field *create_tmp_field_for_schema(THD *thd __attribute__((unused)),
+                                   Item *item, TABLE *table)
+{
+  if (item->field_type() == DRIZZLE_TYPE_VARCHAR)
+  {
+    Field *field;
+    if (item->max_length > MAX_FIELD_VARCHARLENGTH)
+      field= new Field_blob(item->max_length, item->maybe_null,
+                            item->name, item->collation.collation);
+    else
+      field= new Field_varstring(item->max_length, item->maybe_null,
+                                 item->name,
+                                 table->s, item->collation.collation);
+    if (field)
+      field->init(table);
+    return field;
+  }
+  return item->tmp_table_field_from_field_type(table, 0);
+}
+
+
+/**
+  Create field for temporary table.
+
+  @param thd		Thread handler
+  @param table		Temporary table
+  @param item		Item to create a field for
+  @param type		Type of item (normally item->type)
+  @param copy_func	If set and item is a function, store copy of item
+                       in this array
+  @param from_field    if field will be created using other field as example,
+                       pointer example field will be written here
+  @param default_field	If field has a default value field, store it here
+  @param group		1 if we are going to do a relative group by on result
+  @param modify_item	1 if item->result_field should point to new item.
+                       This is relevent for how fill_record() is going to
+                       work:
+                       If modify_item is 1 then fill_record() will update
+                       the record in the original table.
+                       If modify_item is 0 then fill_record() will update
+                       the temporary table
+  @param convert_blob_length If >0 create a varstring(convert_blob_length)
+                             field instead of blob.
+
+  @retval
+    0			on error
+  @retval
+    new_created field
+*/
+
+Field *create_tmp_field(THD *thd, TABLE *table,Item *item, Item::Type type,
+                        Item ***copy_func, Field **from_field,
+                        Field **default_field,
+                        bool group, bool modify_item,
+                        bool table_cant_handle_bit_fields __attribute__((unused)),
+                        bool make_copy_field,
+                        uint convert_blob_length)
+{
+  Field *result;
+  Item::Type orig_type= type;
+  Item *orig_item= 0;
+
+  if (type != Item::FIELD_ITEM &&
+      item->real_item()->type() == Item::FIELD_ITEM)
+  {
+    orig_item= item;
+    item= item->real_item();
+    type= Item::FIELD_ITEM;
+  }
+
+  switch (type) {
+  case Item::SUM_FUNC_ITEM:
+  {
+    Item_sum *item_sum=(Item_sum*) item;
+    result= item_sum->create_tmp_field(group, table, convert_blob_length);
+    if (!result)
+      my_error(ER_OUT_OF_RESOURCES, MYF(ME_FATALERROR));
+    return result;
+  }
+  case Item::FIELD_ITEM:
+  case Item::DEFAULT_VALUE_ITEM:
+  {
+    Item_field *field= (Item_field*) item;
+    bool orig_modify= modify_item;
+    if (orig_type == Item::REF_ITEM)
+      modify_item= 0;
+    /*
+      If item have to be able to store NULLs but underlaid field can't do it,
+      create_tmp_field_from_field() can't be used for tmp field creation.
+    */
+    if (field->maybe_null && !field->field->maybe_null())
+    {
+      result= create_tmp_field_from_item(thd, item, table, NULL,
+                                         modify_item, convert_blob_length);
+      *from_field= field->field;
+      if (result && modify_item)
+        field->result_field= result;
+    } 
+    else
+      result= create_tmp_field_from_field(thd, (*from_field= field->field),
+                                          orig_item ? orig_item->name :
+                                          item->name,
+                                          table,
+                                          modify_item ? field :
+                                          NULL,
+                                          convert_blob_length);
+    if (orig_type == Item::REF_ITEM && orig_modify)
+      ((Item_ref*)orig_item)->set_result_field(result);
+    if (field->field->eq_def(result))
+      *default_field= field->field;
+    return result;
+  }
+  /* Fall through */
+  case Item::FUNC_ITEM:
+    /* Fall through */
+  case Item::COND_ITEM:
+  case Item::FIELD_AVG_ITEM:
+  case Item::FIELD_STD_ITEM:
+  case Item::SUBSELECT_ITEM:
+    /* The following can only happen with 'CREATE TABLE ... SELECT' */
+  case Item::PROC_ITEM:
+  case Item::INT_ITEM:
+  case Item::REAL_ITEM:
+  case Item::DECIMAL_ITEM:
+  case Item::STRING_ITEM:
+  case Item::REF_ITEM:
+  case Item::NULL_ITEM:
+  case Item::VARBIN_ITEM:
+    if (make_copy_field)
+    {
+      assert(((Item_result_field*)item)->result_field);
+      *from_field= ((Item_result_field*)item)->result_field;
+    }
+    return create_tmp_field_from_item(thd, item, table,
+                                      (make_copy_field ? 0 : copy_func),
+                                       modify_item, convert_blob_length);
+  case Item::TYPE_HOLDER:  
+    result= ((Item_type_holder *)item)->make_field_by_type(table);
+    result->set_derivation(item->collation.derivation);
+    return result;
+  default:					// Dosen't have to be stored
+    return 0;
+  }
+}
+
+/**
+  Create a temp table according to a field list.
+
+  Given field pointers are changed to point at tmp_table for
+  send_fields. The table object is self contained: it's
+  allocated in its own memory root, as well as Field objects
+  created for table columns.
+  This function will replace Item_sum items in 'fields' list with
+  corresponding Item_field items, pointing at the fields in the
+  temporary table, unless this was prohibited by true
+  value of argument save_sum_fields. The Item_field objects
+  are created in THD memory root.
+
+  @param thd                  thread handle
+  @param param                a description used as input to create the table
+  @param fields               list of items that will be used to define
+                              column types of the table (also see NOTES)
+  @param group                TODO document
+  @param distinct             should table rows be distinct
+  @param save_sum_fields      see NOTES
+  @param select_options
+  @param rows_limit
+  @param table_alias          possible name of the temporary table that can
+                              be used for name resolving; can be "".
+*/
+
+#define STRING_TOTAL_LENGTH_TO_PACK_ROWS 128
+#define AVG_STRING_LENGTH_TO_PACK_ROWS   64
+#define RATIO_TO_PACK_ROWS	       2
+#define MIN_STRING_LENGTH_TO_PACK_ROWS   10
+
+TABLE *
+create_tmp_table(THD *thd,TMP_TABLE_PARAM *param,List<Item> &fields,
+		 ORDER *group, bool distinct, bool save_sum_fields,
+		 uint64_t select_options, ha_rows rows_limit,
+		 char *table_alias)
+{
+  MEM_ROOT *mem_root_save, own_root;
+  TABLE *table;
+  TABLE_SHARE *share;
+  uint	i,field_count,null_count,null_pack_length;
+  uint  copy_func_count= param->func_count;
+  uint  hidden_null_count, hidden_null_pack_length, hidden_field_count;
+  uint  blob_count,group_null_items, string_count;
+  uint  temp_pool_slot=MY_BIT_NONE;
+  uint fieldnr= 0;
+  ulong reclength, string_total_length;
+  bool  using_unique_constraint= 0;
+  bool  use_packed_rows= 0;
+  bool  not_all_columns= !(select_options & TMP_TABLE_ALL_COLUMNS);
+  char  *tmpname,path[FN_REFLEN];
+  uchar	*pos, *group_buff, *bitmaps;
+  uchar *null_flags;
+  Field **reg_field, **from_field, **default_field;
+  uint *blob_field;
+  Copy_field *copy=0;
+  KEY *keyinfo;
+  KEY_PART_INFO *key_part_info;
+  Item **copy_func;
+  MI_COLUMNDEF *recinfo;
+  uint total_uneven_bit_length= 0;
+  bool force_copy_fields= param->force_copy_fields;
+
+  status_var_increment(thd->status_var.created_tmp_tables);
+
+  if (use_temp_pool && !(test_flags & TEST_KEEP_TMP_TABLES))
+    temp_pool_slot = bitmap_lock_set_next(&temp_pool);
+
+  if (temp_pool_slot != MY_BIT_NONE) // we got a slot
+    sprintf(path, "%s_%lx_%i", tmp_file_prefix,
+            current_pid, temp_pool_slot);
+  else
+  {
+    /* if we run out of slots or we are not using tempool */
+    sprintf(path,"%s%lx_%lx_%x", tmp_file_prefix,current_pid,
+            thd->thread_id, thd->tmp_table++);
+  }
+
+  /*
+    No need to change table name to lower case as we are only creating
+    MyISAM or HEAP tables here
+  */
+  fn_format(path, path, mysql_tmpdir, "", MY_REPLACE_EXT|MY_UNPACK_FILENAME);
+
+
+  if (group)
+  {
+    if (!param->quick_group)
+      group=0;					// Can't use group key
+    else for (ORDER *tmp=group ; tmp ; tmp=tmp->next)
+    {
+      /*
+        marker == 4 means two things:
+        - store NULLs in the key, and
+        - convert BIT fields to 64-bit long, needed because MEMORY tables
+          can't index BIT fields.
+      */
+      (*tmp->item)->marker= 4;
+      if ((*tmp->item)->max_length >= CONVERT_IF_BIGGER_TO_BLOB)
+	using_unique_constraint=1;
+    }
+    if (param->group_length >= MAX_BLOB_WIDTH)
+      using_unique_constraint=1;
+    if (group)
+      distinct=0;				// Can't use distinct
+  }
+
+  field_count=param->field_count+param->func_count+param->sum_func_count;
+  hidden_field_count=param->hidden_field_count;
+
+  /*
+    When loose index scan is employed as access method, it already
+    computes all groups and the result of all aggregate functions. We
+    make space for the items of the aggregate function in the list of
+    functions TMP_TABLE_PARAM::items_to_copy, so that the values of
+    these items are stored in the temporary table.
+  */
+  if (param->precomputed_group_by)
+    copy_func_count+= param->sum_func_count;
+  
+  init_sql_alloc(&own_root, TABLE_ALLOC_BLOCK_SIZE, 0);
+
+  if (!multi_alloc_root(&own_root,
+                        &table, sizeof(*table),
+                        &share, sizeof(*share),
+                        &reg_field, sizeof(Field*) * (field_count+1),
+                        &default_field, sizeof(Field*) * (field_count),
+                        &blob_field, sizeof(uint)*(field_count+1),
+                        &from_field, sizeof(Field*)*field_count,
+                        &copy_func, sizeof(*copy_func)*(copy_func_count+1),
+                        &param->keyinfo, sizeof(*param->keyinfo),
+                        &key_part_info,
+                        sizeof(*key_part_info)*(param->group_parts+1),
+                        &param->start_recinfo,
+                        sizeof(*param->recinfo)*(field_count*2+4),
+                        &tmpname, (uint) strlen(path)+1,
+                        &group_buff, (group && ! using_unique_constraint ?
+                                      param->group_length : 0),
+                        &bitmaps, bitmap_buffer_size(field_count)*2,
+                        NullS))
+  {
+    if (temp_pool_slot != MY_BIT_NONE)
+      bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
+    return(NULL);				/* purecov: inspected */
+  }
+  /* Copy_field belongs to TMP_TABLE_PARAM, allocate it in THD mem_root */
+  if (!(param->copy_field= copy= new (thd->mem_root) Copy_field[field_count]))
+  {
+    if (temp_pool_slot != MY_BIT_NONE)
+      bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
+    free_root(&own_root, MYF(0));               /* purecov: inspected */
+    return(NULL);				/* purecov: inspected */
+  }
+  param->items_to_copy= copy_func;
+  stpcpy(tmpname,path);
+  /* make table according to fields */
+
+  memset(table, 0, sizeof(*table));
+  memset(reg_field, 0, sizeof(Field*)*(field_count+1));
+  memset(default_field, 0, sizeof(Field*) * (field_count));
+  memset(from_field, 0, sizeof(Field*)*field_count);
+
+  table->mem_root= own_root;
+  mem_root_save= thd->mem_root;
+  thd->mem_root= &table->mem_root;
+
+  table->field=reg_field;
+  table->alias= table_alias;
+  table->reginfo.lock_type=TL_WRITE;	/* Will be updated */
+  table->db_stat=HA_OPEN_KEYFILE+HA_OPEN_RNDFILE;
+  table->map=1;
+  table->temp_pool_slot = temp_pool_slot;
+  table->copy_blobs= 1;
+  table->in_use= thd;
+  table->quick_keys.init();
+  table->covering_keys.init();
+  table->keys_in_use_for_query.init();
+
+  table->setShare(share);
+  init_tmp_table_share(thd, share, "", 0, tmpname, tmpname);
+  share->blob_field= blob_field;
+  share->blob_ptr_size= portable_sizeof_char_ptr;
+  share->db_low_byte_first=1;                // True for HEAP and MyISAM
+  share->table_charset= param->table_charset;
+  share->primary_key= MAX_KEY;               // Indicate no primary key
+  share->keys_for_keyread.init();
+  share->keys_in_use.init();
+
+  /* Calculate which type of fields we will store in the temporary table */
+
+  reclength= string_total_length= 0;
+  blob_count= string_count= null_count= hidden_null_count= group_null_items= 0;
+  param->using_indirect_summary_function=0;
+
+  List_iterator_fast<Item> li(fields);
+  Item *item;
+  Field **tmp_from_field=from_field;
+  while ((item=li++))
+  {
+    Item::Type type=item->type();
+    if (not_all_columns)
+    {
+      if (item->with_sum_func && type != Item::SUM_FUNC_ITEM)
+      {
+        if (item->used_tables() & OUTER_REF_TABLE_BIT)
+          item->update_used_tables();
+        if (type == Item::SUBSELECT_ITEM ||
+            (item->used_tables() & ~OUTER_REF_TABLE_BIT))
+        {
+	  /*
+	    Mark that the we have ignored an item that refers to a summary
+	    function. We need to know this if someone is going to use
+	    DISTINCT on the result.
+	  */
+	  param->using_indirect_summary_function=1;
+	  continue;
+        }
+      }
+      if (item->const_item() && (int) hidden_field_count <= 0)
+        continue; // We don't have to store this
+    }
+    if (type == Item::SUM_FUNC_ITEM && !group && !save_sum_fields)
+    {						/* Can't calc group yet */
+      ((Item_sum*) item)->result_field=0;
+      for (i=0 ; i < ((Item_sum*) item)->arg_count ; i++)
+      {
+	Item **argp= ((Item_sum*) item)->args + i;
+	Item *arg= *argp;
+	if (!arg->const_item())
+	{
+	  Field *new_field=
+            create_tmp_field(thd, table, arg, arg->type(), &copy_func,
+                             tmp_from_field, &default_field[fieldnr],
+                             group != 0,not_all_columns,
+                             distinct, 0,
+                             param->convert_blob_length);
+	  if (!new_field)
+	    goto err;					// Should be OOM
+	  tmp_from_field++;
+	  reclength+=new_field->pack_length();
+	  if (new_field->flags & BLOB_FLAG)
+	  {
+	    *blob_field++= fieldnr;
+	    blob_count++;
+	  }
+	  *(reg_field++)= new_field;
+          if (new_field->real_type() == DRIZZLE_TYPE_VARCHAR)
+          {
+            string_count++;
+            string_total_length+= new_field->pack_length();
+          }
+          thd->mem_root= mem_root_save;
+          thd->change_item_tree(argp, new Item_field(new_field));
+          thd->mem_root= &table->mem_root;
+	  if (!(new_field->flags & NOT_NULL_FLAG))
+          {
+	    null_count++;
+            /*
+              new_field->maybe_null() is still false, it will be
+              changed below. But we have to setup Item_field correctly
+            */
+            (*argp)->maybe_null=1;
+          }
+          new_field->field_index= fieldnr++;
+	}
+      }
+    }
+    else
+    {
+      /*
+	The last parameter to create_tmp_field() is a bit tricky:
+
+	We need to set it to 0 in union, to get fill_record() to modify the
+	temporary table.
+	We need to set it to 1 on multi-table-update and in select to
+	write rows to the temporary table.
+	We here distinguish between UNION and multi-table-updates by the fact
+	that in the later case group is set to the row pointer.
+      */
+      Field *new_field= (param->schema_table) ?
+        create_tmp_field_for_schema(thd, item, table) :
+        create_tmp_field(thd, table, item, type, &copy_func,
+                         tmp_from_field, &default_field[fieldnr],
+                         group != 0,
+                         !force_copy_fields &&
+                           (not_all_columns || group !=0),
+                         /*
+                           If item->marker == 4 then we force create_tmp_field
+                           to create a 64-bit longs for BIT fields because HEAP
+                           tables can't index BIT fields directly. We do the same
+                           for distinct, as we want the distinct index to be
+                           usable in this case too.
+                         */
+                         item->marker == 4 || param->bit_fields_as_long,
+                         force_copy_fields,
+                         param->convert_blob_length);
+
+      if (!new_field)
+      {
+	if (thd->is_fatal_error)
+	  goto err;				// Got OOM
+	continue;				// Some kindf of const item
+      }
+      if (type == Item::SUM_FUNC_ITEM)
+	((Item_sum *) item)->result_field= new_field;
+      tmp_from_field++;
+      reclength+=new_field->pack_length();
+      if (!(new_field->flags & NOT_NULL_FLAG))
+	null_count++;
+      if (new_field->flags & BLOB_FLAG)
+      {
+        *blob_field++= fieldnr;
+	blob_count++;
+      }
+      if (item->marker == 4 && item->maybe_null)
+      {
+	group_null_items++;
+	new_field->flags|= GROUP_FLAG;
+      }
+      new_field->field_index= fieldnr++;
+      *(reg_field++)= new_field;
+    }
+    if (!--hidden_field_count)
+    {
+      /*
+        This was the last hidden field; Remember how many hidden fields could
+        have null
+      */
+      hidden_null_count=null_count;
+      /*
+	We need to update hidden_field_count as we may have stored group
+	functions with constant arguments
+      */
+      param->hidden_field_count= fieldnr;
+      null_count= 0;
+    }
+  }
+  assert(fieldnr == (uint) (reg_field - table->field));
+  assert(field_count >= (uint) (reg_field - table->field));
+  field_count= fieldnr;
+  *reg_field= 0;
+  *blob_field= 0;				// End marker
+  share->fields= field_count;
+
+  /* If result table is small; use a heap */
+  /* future: storage engine selection can be made dynamic? */
+  if (blob_count || using_unique_constraint ||
+      (select_options & (OPTION_BIG_TABLES | SELECT_SMALL_RESULT)) ==
+      OPTION_BIG_TABLES || (select_options & TMP_TABLE_FORCE_MYISAM))
+  {
+    share->db_plugin= ha_lock_engine(0, myisam_hton);
+    table->file= get_new_handler(share, &table->mem_root,
+                                 share->db_type());
+    if (group &&
+	(param->group_parts > table->file->max_key_parts() ||
+	 param->group_length > table->file->max_key_length()))
+      using_unique_constraint=1;
+  }
+  else
+  {
+    share->db_plugin= ha_lock_engine(0, heap_hton);
+    table->file= get_new_handler(share, &table->mem_root,
+                                 share->db_type());
+  }
+  if (!table->file)
+    goto err;
+
+
+  if (!using_unique_constraint)
+    reclength+= group_null_items;	// null flag is stored separately
+
+  share->blob_fields= blob_count;
+  if (blob_count == 0)
+  {
+    /* We need to ensure that first byte is not 0 for the delete link */
+    if (param->hidden_field_count)
+      hidden_null_count++;
+    else
+      null_count++;
+  }
+  hidden_null_pack_length=(hidden_null_count+7)/8;
+  null_pack_length= (hidden_null_pack_length +
+                     (null_count + total_uneven_bit_length + 7) / 8);
+  reclength+=null_pack_length;
+  if (!reclength)
+    reclength=1;				// Dummy select
+  /* Use packed rows if there is blobs or a lot of space to gain */
+  if (blob_count || ((string_total_length >= STRING_TOTAL_LENGTH_TO_PACK_ROWS) && (reclength / string_total_length <= RATIO_TO_PACK_ROWS || (string_total_length / string_count) >= AVG_STRING_LENGTH_TO_PACK_ROWS)))
+    use_packed_rows= 1;
+
+  share->reclength= reclength;
+  {
+    uint alloc_length=ALIGN_SIZE(reclength+MI_UNIQUE_HASH_LENGTH+1);
+    share->rec_buff_length= alloc_length;
+    if (!(table->record[0]= (uchar*)
+                            alloc_root(&table->mem_root, alloc_length*3)))
+      goto err;
+    table->record[1]= table->record[0]+alloc_length;
+    share->default_values= table->record[1]+alloc_length;
+  }
+  copy_func[0]=0;				// End marker
+  param->func_count= copy_func - param->items_to_copy; 
+
+  table->setup_tmp_table_column_bitmaps(bitmaps);
+
+  recinfo=param->start_recinfo;
+  null_flags=(uchar*) table->record[0];
+  pos=table->record[0]+ null_pack_length;
+  if (null_pack_length)
+  {
+    memset(recinfo, 0, sizeof(*recinfo));
+    recinfo->type=FIELD_NORMAL;
+    recinfo->length=null_pack_length;
+    recinfo++;
+    memset(null_flags, 255, null_pack_length);	// Set null fields
+
+    table->null_flags= (uchar*) table->record[0];
+    share->null_fields= null_count+ hidden_null_count;
+    share->null_bytes= null_pack_length;
+  }
+  null_count= (blob_count == 0) ? 1 : 0;
+  hidden_field_count=param->hidden_field_count;
+  for (i=0,reg_field=table->field; i < field_count; i++,reg_field++,recinfo++)
+  {
+    Field *field= *reg_field;
+    uint length;
+    memset(recinfo, 0, sizeof(*recinfo));
+
+    if (!(field->flags & NOT_NULL_FLAG))
+    {
+      if (field->flags & GROUP_FLAG && !using_unique_constraint)
+      {
+	/*
+	  We have to reserve one byte here for NULL bits,
+	  as this is updated by 'end_update()'
+	*/
+	*pos++=0;				// Null is stored here
+	recinfo->length=1;
+	recinfo->type=FIELD_NORMAL;
+	recinfo++;
+	memset(recinfo, 0, sizeof(*recinfo));
+      }
+      else
+      {
+	recinfo->null_bit= 1 << (null_count & 7);
+	recinfo->null_pos= null_count/8;
+      }
+      field->move_field(pos,null_flags+null_count/8,
+			1 << (null_count & 7));
+      null_count++;
+    }
+    else
+      field->move_field(pos,(uchar*) 0,0);
+    field->reset();
+
+    /*
+      Test if there is a default field value. The test for ->ptr is to skip
+      'offset' fields generated by initalize_tables
+    */
+    if (default_field[i] && default_field[i]->ptr)
+    {
+      /* 
+         default_field[i] is set only in the cases  when 'field' can
+         inherit the default value that is defined for the field referred
+         by the Item_field object from which 'field' has been created.
+      */
+      my_ptrdiff_t diff;
+      Field *orig_field= default_field[i];
+      /* Get the value from default_values */
+      diff= (my_ptrdiff_t) (orig_field->table->s->default_values-
+                            orig_field->table->record[0]);
+      orig_field->move_field_offset(diff);      // Points now at default_values
+      if (orig_field->is_real_null())
+        field->set_null();
+      else
+      {
+        field->set_notnull();
+        memcpy(field->ptr, orig_field->ptr, field->pack_length());
+      }
+      orig_field->move_field_offset(-diff);     // Back to record[0]
+    } 
+
+    if (from_field[i])
+    {						/* Not a table Item */
+      copy->set(field,from_field[i],save_sum_fields);
+      copy++;
+    }
+    length=field->pack_length();
+    pos+= length;
+
+    /* Make entry for create table */
+    recinfo->length=length;
+    if (field->flags & BLOB_FLAG)
+      recinfo->type= (int) FIELD_BLOB;
+    else
+      recinfo->type=FIELD_NORMAL;
+    if (!--hidden_field_count)
+      null_count=(null_count+7) & ~7;		// move to next byte
+
+    // fix table name in field entry
+    field->table_name= &table->alias;
+  }
+
+  param->copy_field_end=copy;
+  param->recinfo=recinfo;
+  store_record(table,s->default_values);        // Make empty default record
+
+  if (thd->variables.tmp_table_size == ~ (uint64_t) 0)		// No limit
+    share->max_rows= ~(ha_rows) 0;
+  else
+    share->max_rows= (ha_rows) (((share->db_type() == heap_hton) ?
+                                 min(thd->variables.tmp_table_size,
+                                     thd->variables.max_heap_table_size) :
+                                 thd->variables.tmp_table_size) /
+			         share->reclength);
+  set_if_bigger(share->max_rows,1);		// For dummy start options
+  /*
+    Push the LIMIT clause to the temporary table creation, so that we
+    materialize only up to 'rows_limit' records instead of all result records.
+  */
+  set_if_smaller(share->max_rows, rows_limit);
+  param->end_write_records= rows_limit;
+
+  keyinfo= param->keyinfo;
+
+  if (group)
+  {
+    table->group=group;				/* Table is grouped by key */
+    param->group_buff=group_buff;
+    share->keys=1;
+    share->uniques= test(using_unique_constraint);
+    table->key_info=keyinfo;
+    keyinfo->key_part=key_part_info;
+    keyinfo->flags=HA_NOSAME;
+    keyinfo->usable_key_parts=keyinfo->key_parts= param->group_parts;
+    keyinfo->key_length=0;
+    keyinfo->rec_per_key=0;
+    keyinfo->algorithm= HA_KEY_ALG_UNDEF;
+    keyinfo->name= (char*) "group_key";
+    ORDER *cur_group= group;
+    for (; cur_group ; cur_group= cur_group->next, key_part_info++)
+    {
+      Field *field=(*cur_group->item)->get_tmp_table_field();
+      bool maybe_null=(*cur_group->item)->maybe_null;
+      key_part_info->null_bit=0;
+      key_part_info->field=  field;
+      key_part_info->offset= field->offset(table->record[0]);
+      key_part_info->length= (uint16_t) field->key_length();
+      key_part_info->type=   (uint8_t) field->key_type();
+      key_part_info->key_type =
+	((ha_base_keytype) key_part_info->type == HA_KEYTYPE_TEXT ||
+	 (ha_base_keytype) key_part_info->type == HA_KEYTYPE_VARTEXT1 ||
+	 (ha_base_keytype) key_part_info->type == HA_KEYTYPE_VARTEXT2) ?
+	0 : FIELDFLAG_BINARY;
+      if (!using_unique_constraint)
+      {
+	cur_group->buff=(char*) group_buff;
+	if (!(cur_group->field= field->new_key_field(thd->mem_root,table,
+                                                     group_buff +
+                                                     test(maybe_null),
+                                                     field->null_ptr,
+                                                     field->null_bit)))
+	  goto err; /* purecov: inspected */
+	if (maybe_null)
+	{
+	  /*
+	    To be able to group on NULL, we reserved place in group_buff
+	    for the NULL flag just before the column. (see above).
+	    The field data is after this flag.
+	    The NULL flag is updated in 'end_update()' and 'end_write()'
+	  */
+	  keyinfo->flags|= HA_NULL_ARE_EQUAL;	// def. that NULL == NULL
+	  key_part_info->null_bit=field->null_bit;
+	  key_part_info->null_offset= (uint) (field->null_ptr -
+					      (uchar*) table->record[0]);
+          cur_group->buff++;                        // Pointer to field data
+	  group_buff++;                         // Skipp null flag
+	}
+        /* In GROUP BY 'a' and 'a ' are equal for VARCHAR fields */
+        key_part_info->key_part_flag|= HA_END_SPACE_ARE_EQUAL;
+	group_buff+= cur_group->field->pack_length();
+      }
+      keyinfo->key_length+=  key_part_info->length;
+    }
+  }
+
+  if (distinct && field_count != param->hidden_field_count)
+  {
+    /*
+      Create an unique key or an unique constraint over all columns
+      that should be in the result.  In the temporary table, there are
+      'param->hidden_field_count' extra columns, whose null bits are stored
+      in the first 'hidden_null_pack_length' bytes of the row.
+    */
+    if (blob_count)
+    {
+      /*
+        Special mode for index creation in MyISAM used to support unique
+        indexes on blobs with arbitrary length. Such indexes cannot be
+        used for lookups.
+      */
+      share->uniques= 1;
+    }
+    null_pack_length-=hidden_null_pack_length;
+    keyinfo->key_parts= ((field_count-param->hidden_field_count)+
+			 (share->uniques ? test(null_pack_length) : 0));
+    table->distinct= 1;
+    share->keys= 1;
+    if (!(key_part_info= (KEY_PART_INFO*)
+          alloc_root(&table->mem_root,
+                     keyinfo->key_parts * sizeof(KEY_PART_INFO))))
+      goto err;
+    memset(key_part_info, 0, keyinfo->key_parts * sizeof(KEY_PART_INFO));
+    table->key_info=keyinfo;
+    keyinfo->key_part=key_part_info;
+    keyinfo->flags=HA_NOSAME | HA_NULL_ARE_EQUAL;
+    keyinfo->key_length=(uint16_t) reclength;
+    keyinfo->name= (char*) "distinct_key";
+    keyinfo->algorithm= HA_KEY_ALG_UNDEF;
+    keyinfo->rec_per_key=0;
+
+    /*
+      Create an extra field to hold NULL bits so that unique indexes on
+      blobs can distinguish NULL from 0. This extra field is not needed
+      when we do not use UNIQUE indexes for blobs.
+    */
+    if (null_pack_length && share->uniques)
+    {
+      key_part_info->null_bit=0;
+      key_part_info->offset=hidden_null_pack_length;
+      key_part_info->length=null_pack_length;
+      key_part_info->field= new Field_varstring(table->record[0],
+                                                (uint32_t) key_part_info->length,
+                                                0,
+                                                (uchar*) 0,
+                                                (uint) 0,
+                                                Field::NONE,
+                                                NullS, 
+                                                table->s,
+                                                &my_charset_bin);
+      if (!key_part_info->field)
+        goto err;
+      key_part_info->field->init(table);
+      key_part_info->key_type=FIELDFLAG_BINARY;
+      key_part_info->type=    HA_KEYTYPE_BINARY;
+      key_part_info++;
+    }
+    /* Create a distinct key over the columns we are going to return */
+    for (i=param->hidden_field_count, reg_field=table->field + i ;
+	 i < field_count;
+	 i++, reg_field++, key_part_info++)
+    {
+      key_part_info->null_bit=0;
+      key_part_info->field=    *reg_field;
+      key_part_info->offset=   (*reg_field)->offset(table->record[0]);
+      key_part_info->length=   (uint16_t) (*reg_field)->pack_length();
+      /* TODO:
+        The below method of computing the key format length of the
+        key part is a copy/paste from opt_range.cc, and table.cc.
+        This should be factored out, e.g. as a method of Field.
+        In addition it is not clear if any of the Field::*_length
+        methods is supposed to compute the same length. If so, it
+        might be reused.
+      */
+      key_part_info->store_length= key_part_info->length;
+
+      if ((*reg_field)->real_maybe_null())
+        key_part_info->store_length+= HA_KEY_NULL_LENGTH;
+      if ((*reg_field)->type() == DRIZZLE_TYPE_BLOB || 
+          (*reg_field)->real_type() == DRIZZLE_TYPE_VARCHAR)
+        key_part_info->store_length+= HA_KEY_BLOB_LENGTH;
+
+      key_part_info->type=     (uint8_t) (*reg_field)->key_type();
+      key_part_info->key_type =
+	((ha_base_keytype) key_part_info->type == HA_KEYTYPE_TEXT ||
+	 (ha_base_keytype) key_part_info->type == HA_KEYTYPE_VARTEXT1 ||
+	 (ha_base_keytype) key_part_info->type == HA_KEYTYPE_VARTEXT2) ?
+	0 : FIELDFLAG_BINARY;
+    }
+  }
+
+  if (thd->is_fatal_error)				// If end of memory
+    goto err;					 /* purecov: inspected */
+  share->db_record_offset= 1;
+  if (share->db_type() == myisam_hton)
+  {
+    if (table->create_myisam_tmp_table(param->keyinfo, param->start_recinfo,
+				       &param->recinfo, select_options))
+      goto err;
+  }
+  if (table->open_tmp_table())
+    goto err;
+
+  thd->mem_root= mem_root_save;
+
+  return(table);
+
+err:
+  thd->mem_root= mem_root_save;
+  table->free_tmp_table(thd);                    /* purecov: inspected */
+  if (temp_pool_slot != MY_BIT_NONE)
+    bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
+  return(NULL);				/* purecov: inspected */
+}
+
+/****************************************************************************/
+
+/**
+  Create a reduced TABLE object with properly set up Field list from a
+  list of field definitions.
+
+    The created table doesn't have a table handler associated with
+    it, has no keys, no group/distinct, no copy_funcs array.
+    The sole purpose of this TABLE object is to use the power of Field
+    class to read/write data to/from table->record[0]. Then one can store
+    the record in any container (RB tree, hash, etc).
+    The table is created in THD mem_root, so are the table's fields.
+    Consequently, if you don't BLOB fields, you don't need to free it.
+
+  @param thd         connection handle
+  @param field_list  list of column definitions
+
+  @return
+    0 if out of memory, TABLE object in case of success
+*/
+
+TABLE *create_virtual_tmp_table(THD *thd, List<Create_field> &field_list)
+{
+  uint field_count= field_list.elements;
+  uint blob_count= 0;
+  Field **field;
+  Create_field *cdef;                           /* column definition */
+  uint record_length= 0;
+  uint null_count= 0;                 /* number of columns which may be null */
+  uint null_pack_length;              /* NULL representation array length */
+  uint *blob_field;
+  uchar *bitmaps;
+  TABLE *table;
+  TABLE_SHARE *share;
+
+  if (!multi_alloc_root(thd->mem_root,
+                        &table, sizeof(*table),
+                        &share, sizeof(*share),
+                        &field, (field_count + 1) * sizeof(Field*),
+                        &blob_field, (field_count+1) *sizeof(uint),
+                        &bitmaps, bitmap_buffer_size(field_count)*2,
+                        NullS))
+    return 0;
+
+  memset(table, 0, sizeof(*table));
+  memset(share, 0, sizeof(*share));
+  table->field= field;
+  table->s= share;
+  share->blob_field= blob_field;
+  share->fields= field_count;
+  share->blob_ptr_size= portable_sizeof_char_ptr;
+  table->setup_tmp_table_column_bitmaps(bitmaps);
+
+  /* Create all fields and calculate the total length of record */
+  List_iterator_fast<Create_field> it(field_list);
+  while ((cdef= it++))
+  {
+    *field= make_field(share, 0, cdef->length,
+                       (uchar*) (f_maybe_null(cdef->pack_flag) ? "" : 0),
+                       f_maybe_null(cdef->pack_flag) ? 1 : 0,
+                       cdef->pack_flag, cdef->sql_type, cdef->charset,
+                       cdef->unireg_check,
+                       cdef->interval, cdef->field_name);
+    if (!*field)
+      goto error;
+    (*field)->init(table);
+    record_length+= (*field)->pack_length();
+    if (! ((*field)->flags & NOT_NULL_FLAG))
+      null_count++;
+
+    if ((*field)->flags & BLOB_FLAG)
+      share->blob_field[blob_count++]= (uint) (field - table->field);
+
+    field++;
+  }
+  *field= NULL;                             /* mark the end of the list */
+  share->blob_field[blob_count]= 0;            /* mark the end of the list */
+  share->blob_fields= blob_count;
+
+  null_pack_length= (null_count + 7)/8;
+  share->reclength= record_length + null_pack_length;
+  share->rec_buff_length= ALIGN_SIZE(share->reclength + 1);
+  table->record[0]= (uchar*) thd->alloc(share->rec_buff_length);
+  if (!table->record[0])
+    goto error;
+
+  if (null_pack_length)
+  {
+    table->null_flags= (uchar*) table->record[0];
+    share->null_fields= null_count;
+    share->null_bytes= null_pack_length;
+  }
+
+  table->in_use= thd;           /* field->reset() may access table->in_use */
+  {
+    /* Set up field pointers */
+    uchar *null_pos= table->record[0];
+    uchar *field_pos= null_pos + share->null_bytes;
+    uint null_bit= 1;
+
+    for (field= table->field; *field; ++field)
+    {
+      Field *cur_field= *field;
+      if ((cur_field->flags & NOT_NULL_FLAG))
+        cur_field->move_field(field_pos);
+      else
+      {
+        cur_field->move_field(field_pos, (uchar*) null_pos, null_bit);
+        null_bit<<= 1;
+        if (null_bit == (1 << 8))
+        {
+          ++null_pos;
+          null_bit= 1;
+        }
+      }
+      cur_field->reset();
+
+      field_pos+= cur_field->pack_length();
+    }
+  }
+  return table;
+error:
+  for (field= table->field; *field; ++field)
+    delete *field;                         /* just invokes field destructor */
+  return 0;
+}
+
+
+bool Table::open_tmp_table()
+{
+  int error;
+  if ((error=file->ha_open(this, s->table_name.str,O_RDWR,
+                                  HA_OPEN_TMP_TABLE | HA_OPEN_INTERNAL_TABLE)))
+  {
+    file->print_error(error,MYF(0)); /* purecov: inspected */
+    db_stat=0;
+    return(1);
+  }
+  (void) file->extra(HA_EXTRA_QUICK);		/* Faster */
+  return(0);
+}
+
+
+/*
+  Create MyISAM temporary table
+
+  SYNOPSIS
+    create_myisam_tmp_table()
+      keyinfo         Description of the index (there is always one index)
+      start_recinfo   MyISAM's column descriptions
+      recinfo INOUT   End of MyISAM's column descriptions
+      options         Option bits
+   
+  DESCRIPTION
+    Create a MyISAM temporary table according to passed description. The is
+    assumed to have one unique index or constraint.
+
+    The passed array or MI_COLUMNDEF structures must have this form:
+
+      1. 1-byte column (afaiu for 'deleted' flag) (note maybe not 1-byte
+         when there are many nullable columns)
+      2. Table columns
+      3. One free MI_COLUMNDEF element (*recinfo points here)
+   
+    This function may use the free element to create hash column for unique
+    constraint.
+
+   RETURN
+     false - OK
+     true  - Error
+*/
+
+bool Table::create_myisam_tmp_table(KEY *keyinfo, 
+                                    MI_COLUMNDEF *start_recinfo,
+                                    MI_COLUMNDEF **recinfo, 
+				    uint64_t options)
+{
+  int error;
+  MI_KEYDEF keydef;
+  MI_UNIQUEDEF uniquedef;
+  TABLE_SHARE *share= s;
+
+  if (share->keys)
+  {						// Get keys for ni_create
+    bool using_unique_constraint=0;
+    HA_KEYSEG *seg= (HA_KEYSEG*) alloc_root(&this->mem_root,
+                                            sizeof(*seg) * keyinfo->key_parts);
+    if (!seg)
+      goto err;
+
+    memset(seg, 0, sizeof(*seg) * keyinfo->key_parts);
+    if (keyinfo->key_length >= file->max_key_length() ||
+	keyinfo->key_parts > file->max_key_parts() ||
+	share->uniques)
+    {
+      /* Can't create a key; Make a unique constraint instead of a key */
+      share->keys=    0;
+      share->uniques= 1;
+      using_unique_constraint=1;
+      memset(&uniquedef, 0, sizeof(uniquedef));
+      uniquedef.keysegs=keyinfo->key_parts;
+      uniquedef.seg=seg;
+      uniquedef.null_are_equal=1;
+
+      /* Create extra column for hash value */
+      memset(*recinfo, 0, sizeof(**recinfo));
+      (*recinfo)->type= FIELD_CHECK;
+      (*recinfo)->length=MI_UNIQUE_HASH_LENGTH;
+      (*recinfo)++;
+      share->reclength+=MI_UNIQUE_HASH_LENGTH;
+    }
+    else
+    {
+      /* Create an unique key */
+      memset(&keydef, 0, sizeof(keydef));
+      keydef.flag=HA_NOSAME | HA_BINARY_PACK_KEY | HA_PACK_KEY;
+      keydef.keysegs=  keyinfo->key_parts;
+      keydef.seg= seg;
+    }
+    for (uint i=0; i < keyinfo->key_parts ; i++,seg++)
+    {
+      Field *field=keyinfo->key_part[i].field;
+      seg->flag=     0;
+      seg->language= field->charset()->number;
+      seg->length=   keyinfo->key_part[i].length;
+      seg->start=    keyinfo->key_part[i].offset;
+      if (field->flags & BLOB_FLAG)
+      {
+	seg->type=
+	((keyinfo->key_part[i].key_type & FIELDFLAG_BINARY) ?
+	 HA_KEYTYPE_VARBINARY2 : HA_KEYTYPE_VARTEXT2);
+	seg->bit_start= (uint8_t)(field->pack_length() - share->blob_ptr_size);
+	seg->flag= HA_BLOB_PART;
+	seg->length=0;			// Whole blob in unique constraint
+      }
+      else
+      {
+	seg->type= keyinfo->key_part[i].type;
+      }
+      if (!(field->flags & NOT_NULL_FLAG))
+      {
+	seg->null_bit= field->null_bit;
+	seg->null_pos= (uint) (field->null_ptr - (uchar*) record[0]);
+	/*
+	  We are using a GROUP BY on something that contains NULL
+	  In this case we have to tell MyISAM that two NULL should
+	  on INSERT be regarded at the same value
+	*/
+	if (!using_unique_constraint)
+	  keydef.flag|= HA_NULL_ARE_EQUAL;
+      }
+    }
+  }
+  MI_CREATE_INFO create_info;
+  memset(&create_info, 0, sizeof(create_info));
+
+  if ((options & (OPTION_BIG_TABLES | SELECT_SMALL_RESULT)) ==
+      OPTION_BIG_TABLES)
+    create_info.data_file_length= ~(uint64_t) 0;
+
+  if ((error=mi_create(share->table_name.str, share->keys, &keydef,
+		       (uint) (*recinfo-start_recinfo),
+		       start_recinfo,
+		       share->uniques, &uniquedef,
+		       &create_info,
+		       HA_CREATE_TMP_TABLE)))
+  {
+    file->print_error(error,MYF(0));	/* purecov: inspected */
+    db_stat=0;
+    goto err;
+  }
+  status_var_increment(in_use->status_var.created_tmp_disk_tables);
+  share->db_record_offset= 1;
+  return false;
+ err:
+  return true;
+}
+
+
+void Table::free_tmp_table(THD *thd)
+{
+  MEM_ROOT own_root= mem_root;
+  const char *save_proc_info;
+
+  save_proc_info=thd->proc_info;
+  thd_proc_info(thd, "removing tmp table");
+
+  if (file)
+  {
+    if (db_stat)
+      file->ha_drop_table(s->table_name.str);
+    else
+      file->ha_delete_table(s->table_name.str);
+    delete file;
+  }
+
+  /* free blobs */
+  for (Field **ptr= field ; *ptr ; ptr++)
+    (*ptr)->free();
+  free_io_cache(this);
+
+  if (temp_pool_slot != MY_BIT_NONE)
+    bitmap_lock_clear_bit(&temp_pool, temp_pool_slot);
+
+  plugin_unlock(0, s->db_plugin);
+
+  free_root(&own_root, MYF(0)); /* the table is allocated in its own root */
+  thd_proc_info(thd, save_proc_info);
+
+  return;
+}
+
+/**
+  If a HEAP table gets full, create a MyISAM table and copy all rows
+  to this.
+*/
+
+bool create_myisam_from_heap(THD *thd, TABLE *table,
+                             MI_COLUMNDEF *start_recinfo,
+                             MI_COLUMNDEF **recinfo, 
+			     int error, bool ignore_last_dupp_key_error)
+{
+  TABLE new_table;
+  TABLE_SHARE share;
+  const char *save_proc_info;
+  int write_err;
+
+  if (table->s->db_type() != heap_hton || 
+      error != HA_ERR_RECORD_FILE_FULL)
+  {
+    table->file->print_error(error,MYF(0));
+    return(1);
+  }
+  new_table= *table;
+  share= *table->s;
+  new_table.s= &share;
+  new_table.s->db_plugin= ha_lock_engine(thd, myisam_hton);
+  if (!(new_table.file= get_new_handler(&share, &new_table.mem_root,
+                                        new_table.s->db_type())))
+    return(1);				// End of memory
+
+  save_proc_info=thd->proc_info;
+  thd_proc_info(thd, "converting HEAP to MyISAM");
+
+  if (new_table.create_myisam_tmp_table(table->key_info, start_recinfo,
+					recinfo, thd->lex->select_lex.options | 
+					thd->options))
+    goto err2;
+  if (new_table.open_tmp_table())
+    goto err1;
+  if (table->file->indexes_are_disabled())
+    new_table.file->ha_disable_indexes(HA_KEY_SWITCH_ALL);
+  table->file->ha_index_or_rnd_end();
+  table->file->ha_rnd_init(1);
+  if (table->no_rows)
+  {
+    new_table.file->extra(HA_EXTRA_NO_ROWS);
+    new_table.no_rows=1;
+  }
+
+#ifdef TO_BE_DONE_LATER_IN_4_1
+  /*
+    To use start_bulk_insert() (which is new in 4.1) we need to find
+    all places where a corresponding end_bulk_insert() should be put.
+  */
+  table->file->info(HA_STATUS_VARIABLE); /* update table->file->stats.records */
+  new_table.file->ha_start_bulk_insert(table->file->stats.records);
+#else
+  /* HA_EXTRA_WRITE_CACHE can stay until close, no need to disable it */
+  new_table.file->extra(HA_EXTRA_WRITE_CACHE);
+#endif
+
+  /*
+    copy all old rows from heap table to MyISAM table
+    This is the only code that uses record[1] to read/write but this
+    is safe as this is a temporary MyISAM table without timestamp/autoincrement.
+  */
+  while (!table->file->rnd_next(new_table.record[1]))
+  {
+    write_err= new_table.file->ha_write_row(new_table.record[1]);
+    if (write_err)
+      goto err;
+  }
+  /* copy row that filled HEAP table */
+  if ((write_err=new_table.file->ha_write_row(table->record[0])))
+  {
+    if (new_table.file->is_fatal_error(write_err, HA_CHECK_DUP) ||
+	!ignore_last_dupp_key_error)
+      goto err;
+  }
+
+  /* remove heap table and change to use myisam table */
+  (void) table->file->ha_rnd_end();
+  (void) table->file->close();                  // This deletes the table !
+  delete table->file;
+  table->file=0;
+  plugin_unlock(0, table->s->db_plugin);
+  share.db_plugin= my_plugin_lock(0, &share.db_plugin);
+  new_table.s= table->s;                       // Keep old share
+  *table= new_table;
+  *table->s= share;
+  
+  table->file->change_table_ptr(table, table->s);
+  table->use_all_columns();
+  if (save_proc_info)
+  {
+    const char *new_proc_info=
+      (!strcmp(save_proc_info,"Copying to tmp table") ?
+      "Copying to tmp table on disk" : save_proc_info);
+    thd_proc_info(thd, new_proc_info);
+  }
+  return(0);
+
+ err:
+  table->file->print_error(write_err, MYF(0));
+  (void) table->file->ha_rnd_end();
+  (void) new_table.file->close();
+ err1:
+  new_table.file->ha_delete_table(new_table.s->table_name.str);
+ err2:
+  delete new_table.file;
+  thd_proc_info(thd, save_proc_info);
+  table->mem_root= new_table.mem_root;
+  return(1);
+}
+
 
 /*****************************************************************************
 ** Instansiate templates

=== modified file 'drizzled/table.h'
--- drizzled/table.h	2008-08-15 15:01:59 +0000
+++ drizzled/table.h	2008-08-15 21:14:46 +0000
@@ -16,6 +16,11 @@
 
 /* Structs that defines the TABLE */
 
+#ifndef DRIZZLED_TABLE_H
+#define DRIZZLED_TABLE_H
+
+#include <storage/myisam/myisam.h>
+
 class Item;				/* Needed by ORDER */
 class Item_subselect;
 class st_select_lex_unit;
@@ -349,6 +354,11 @@
   INDEX_HINT_FORCE
 };
 
+bool create_myisam_from_heap(THD *thd, TABLE *table,
+                             MI_COLUMNDEF *start_recinfo,
+                             MI_COLUMNDEF **recinfo, 
+                             int error, bool ignore_last_dupp_key_error);
+
 class Table {
 public:
   TABLE_SHARE	*s;
@@ -372,6 +382,17 @@
   inline bool isReplaceWithNameLock() { return s->replace_with_name_lock; }
   inline bool isWaitingOnCondition() { return s->waiting_on_cond; }                 /* Protection against free */
 
+  void updateCreateInfo(HA_CREATE_INFO *create_info);
+  void setup_tmp_table_column_bitmaps(uchar *bitmaps);
+
+  /* For TMP tables, should be pulled out as a class */
+  bool create_myisam_tmp_table(KEY *keyinfo, 
+                               MI_COLUMNDEF *start_recinfo,
+                               MI_COLUMNDEF **recinfo, 
+                               uint64_t options);
+  void free_tmp_table(THD *thd);
+  bool open_tmp_table();
+
   handler	*file;
   Table *next, *prev;
 
@@ -1208,3 +1229,4 @@
 
 size_t max_row_length(TABLE *table, const uchar *data);
 
+#endif /* DRIZZLED_TABLE_H */

=== added file 'drizzled/tmp_table.cc'
--- drizzled/tmp_table.cc	1970-01-01 00:00:00 +0000
+++ drizzled/tmp_table.cc	2008-08-15 21:14:46 +0000
@@ -0,0 +1,1 @@
+#include "tmp_table.h"

=== added file 'drizzled/tmp_table.h'
--- drizzled/tmp_table.h	1970-01-01 00:00:00 +0000
+++ drizzled/tmp_table.h	2008-08-15 21:14:46 +0000
@@ -0,0 +1,6 @@
+#include <drizzled/server_includes.h>
+
+#ifndef DRIZZLED_TMP_TABLE_H
+#define DRIZZLED_TMP_TABLE_H
+
+#endif /* DRIZZLED_TMP_TABLE_H */

